/**
 *  This file is part of DJFourierv4 developed by Blai Meléndez Catalán under
 	the supervision of prof. Horst Eidenberger (TU Wien).

    DJFourierv4 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DJFourierv4 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

 	<http://www.gnu.org/licenses/>. */

package com.example.djfourierv4;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.nio.BufferUnderflowException;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.ShortBuffer;

import android.app.Activity;
import android.graphics.Color;
import android.os.Bundle;
import android.os.Environment;
import android.support.v4.view.MotionEventCompat;
import android.view.Menu;
import android.view.MenuItem;
import android.view.MotionEvent;
import android.widget.SeekBar;
import android.widget.SeekBar.OnSeekBarChangeListener;
import android.widget.TextView;

public class Cycle extends Activity implements Runnable{
	
	private int CAPTURE_RATE = 4096;
	private int N = 0;
	private int readingProgress = 0;
	
	private VisualizerView visualizerView;
	
	private TextView textView;
	private TextView textView2;
	private TextView textView3;
	private TextView textView4;
	
	private SeekBar seekBar;
		
	private Manipulation manipulation;
	
	private FFT mFFT;
		
	private WavInfo wavInfo;
	
	private WavReader wavReader;
		
	private CharSequence audioName;
	
	private OutputManager oM;
	
	private Thread thread;
	
	private FileInputStream fis1;
		
	private byte[] header = new byte[44];

	private short[] dataRealOutputShort1 = new short[CAPTURE_RATE];
	private short[] dataRealOutputShort2 = new short[CAPTURE_RATE];
	private byte[] dataRealOutputByte1 = new byte[CAPTURE_RATE];
	private byte[] dataRealOutputByte2 = new byte[CAPTURE_RATE];

	private double[] dataReal1 = new double[CAPTURE_RATE];
	private double[] dataImag1 = new double[CAPTURE_RATE];
	private double[] dataReal2 = new double[CAPTURE_RATE];
	private double[] dataImag2 = new double[CAPTURE_RATE];

	private double[] dataModul1 = new double[CAPTURE_RATE / 2];
	private double[] dataModul2 = new double[CAPTURE_RATE / 2];
	
	private byte[] dataRealInputByteToShort1 = new byte[2 * CAPTURE_RATE];
	private byte[] dataRealInputByteToShort2 = new byte[2 * CAPTURE_RATE];

	private short[] dataRealInputShort1 = new short[CAPTURE_RATE];
	private short[] dataRealInputShort2 = new short[CAPTURE_RATE];

	private short[] lastNShorts = null;
		
	private byte[] dataRealInputByteToByte1 = new byte[CAPTURE_RATE];
	private byte[] dataRealInputByteToByte2 = new byte[CAPTURE_RATE];
	
	private byte[] dataRealInputByte1 = new byte[CAPTURE_RATE];
	private byte[] dataRealInputByte2 = new byte[CAPTURE_RATE];
	
	private byte[] lastNBytes = null;
	
	private boolean shouldContinue = true, isProgressChanged = false;
	
	private Filter filter;
	private FilterView filterView;
	
	private Equalizer equalizer;
	private EqualizerView equalizerView;
	
	private Synthesizer synth;
	
	private LoudnessContour lc;
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);		
		
		setContentView(R.layout.activity_audio_player);
				
		//retrieve the name of the file we want to open.
		audioName = getIntent().getExtras().getCharSequence("audioName");
		
		wavReader = new WavReader();
		
		//Reading of the header of the wave file. WavInfo stores the relevant information.
		try {
			fis1 =  new FileInputStream(Environment.getExternalStorageDirectory().getPath()+ "/Music/" + audioName);
			if(fis1.read(header) == 44){
				wavInfo = wavReader.readHeader(header);
				if(wavInfo.getFinish() == true){
					finish();
				}
				wavInfo.setSize(fis1.available());
			}
		} catch (FileNotFoundException e) {e.printStackTrace();} catch (IOException e) {e.printStackTrace();}
		
		//Initialization of the views and classes with the information obtained from the header.
		if(wavInfo.getFinish() == false){
			
			textView = (TextView) findViewById(R.id.textView1);
			textView2 = (TextView) findViewById(R.id.textView2);
			textView3 = (TextView) findViewById(R.id.textView3);
			textView4 = (TextView) findViewById(R.id.textView4);
			
			visualizerView = (VisualizerView) findViewById(R.id.visualizerView1);
			visualizerView.setParameters(CAPTURE_RATE, getResources().getDisplayMetrics().widthPixels, wavInfo.getRate(), wavInfo.getBits());
			
			filterView = (FilterView) findViewById(R.id.filterView1);
			filterView.setParameters(CAPTURE_RATE, wavInfo.getRate());
			filter = new Filter(CAPTURE_RATE, wavInfo.getRate(), filterView, textView4);
			
			equalizerView = (EqualizerView) findViewById(R.id.equalizerView1);
			equalizerView.setParameters(CAPTURE_RATE);
			equalizer = new Equalizer(CAPTURE_RATE, wavInfo.getRate(), equalizerView);
			
	        synth = new Synthesizer(wavInfo.getRate(), CAPTURE_RATE, wavInfo.getBits(), (double) getResources().getDisplayMetrics().widthPixels);

			mFFT = new FFT(CAPTURE_RATE);
	        
			manipulation = new Manipulation(CAPTURE_RATE, wavInfo.getRate(), wavInfo.getBits(), (double) getResources().getDisplayMetrics().heightPixels, (double) getResources().getDisplayMetrics().widthPixels, visualizerView, textView, textView2, textView3, textView4, filter, equalizer, synth);
			
			lc = new LoudnessContour(wavInfo.getRate(), getResources().getDisplayMetrics().widthPixels);
			
		    //Preparations for the play-back and the visualization 
	        oM = new OutputManager(Cycle.this, wavInfo.getChannels(), wavInfo.getRate(), wavInfo.getBits());
	        
			oM.playAudio();
			
			oM.setPlaybackListener(CAPTURE_RATE, visualizerView, 0);			
	        
			//This thread contains a loop that will read-manipulate-store the data until the wave file is finished.
	        thread = new Thread(this);
			
			thread.start();
		}
	}
	
	/**
	 * The thread containing the loop starts running */
	@Override
	public void run() {
		
		//Seek bar initialization
		seekBar = (SeekBar) findViewById(R.id.seekBar1);
		seekBar.setBackgroundColor(Color.TRANSPARENT);
		seekBar.setIndeterminate(false);
		seekBar.setMax(100);
		
		//We specify how the seek bar must be updated
		seekBar.setOnSeekBarChangeListener(new OnSeekBarChangeListener() {
			
			//It is only when you pull up your finger that the progress is changed
			@Override
			public void onStopTrackingTouch(SeekBar seekBar) {
				
				if(wavInfo.getBits() == 16){
					readingProgress = (int) (((double) seekBar.getProgress() / (4d * 100d)) * (double) wavInfo.getSize());
				}else{
					readingProgress = (int) (((double) seekBar.getProgress() / (2d * 100d)) * (double) wavInfo.getSize());
				}
				isProgressChanged = true;
			}
			
			@Override
			public void onStartTrackingTouch(SeekBar seekBar) {				
			}
			
			//This only modifies the visual position of the seek bar until you pull up your finger.
			@Override
			public void onProgressChanged(SeekBar seekBar, int progress,
					boolean fromUser) {
				
				if(fromUser == true){
					
					seekBar.setProgress(progress);					
				}
			}
		});
		
		//If the wave file has 16 bits per sample...
		if(wavInfo.getBits() == 16){
			
			//START OF THE LOOP
			for(readingProgress = 0; readingProgress < (wavInfo.getSize() / ((Short.SIZE / 8) * 2)); readingProgress += CAPTURE_RATE - N + 1){
				
				//Correction of the readingProgress
				if(N == 0 && readingProgress != 0){
					readingProgress -= 1;
				}
				
				//Until the value of the action bar height is correct.
				while(!manipulation.setActionBarHeight(getActionBar().getHeight()));
												
				//Breaks the loop and therefore ends the thread in case back is pressed on the screen
				if(shouldContinue == false){
					
					try {
						fis1.close();
					} catch (IOException e) {e.printStackTrace();}
					
					break;
				}
				
				//Check the number of samples we have to take from the wave file to correctly compute its convolution with any filter. And change it if necessary.
				if(N != manipulation.getFiltersAndEqualizerMaxN()){
					
					//Get the biggest N for all the filters that are being applied.
					N = manipulation.getFiltersAndEqualizerMaxN();

					//Change parameters for the synchronization between output sound and spectrum visualization.
					oM.setPlaybackListener(CAPTURE_RATE, visualizerView, N);
					
					//Set N for synthesizer.
					synth.setN(N);
					
					//Re-initialization of the arrays. We need the last N - 1 positions to be equal to 0.
					dataRealInputByteToShort1 = new byte[2 * CAPTURE_RATE];
					dataRealInputByteToShort2 = new byte[2 * CAPTURE_RATE];
										
					dataRealInputShort1 = new short[CAPTURE_RATE];
					dataRealInputShort2 = new short[CAPTURE_RATE];
					
					//This vector is where we will save the last N - 1 values of the data to handle the overlap.
					if(N != 0){
						lastNShorts = new short[N - 1];
					}
				}				
				
				//When the seek bar is clicked.
				if(isProgressChanged == true){
					
					//Re-initialization of the input stream.
					try {
						fis1.close();
						fis1 = new FileInputStream(Environment.getExternalStorageDirectory().getPath()+ "/Music/" + audioName);
					} catch (IOException e1) {e1.printStackTrace();	}
					
					//Sets the input stream to the correct place after manipulating the seek bar.
					try {
						fis1.skip((long) (readingProgress * 4d));
					} catch (IOException e) {e.printStackTrace();}
					
					isProgressChanged = false;
				}
				
				
				//Reading of the data. We take two pieces of data at a time.
				try {
					if(N != 0){
						fis1.read(dataRealInputByteToShort1, 0, dataRealInputByteToShort1.length - 2 * (N - 1));
						fis1.read(dataRealInputByteToShort2, 0, dataRealInputByteToShort2.length - 2 * (N - 1));
					}else{
						fis1.read(dataRealInputByteToShort1, 0, dataRealInputByteToShort1.length);
						fis1.read(dataRealInputByteToShort2, 0, dataRealInputByteToShort2.length);
					}
				} catch (IOException e1) {e1.printStackTrace();}

				//Transformation from byte array to short buffer.
				ShortBuffer bufferDataRealShort1 = ByteBuffer.wrap(dataRealInputByteToShort1).order(ByteOrder.LITTLE_ENDIAN).asShortBuffer();
				ShortBuffer bufferDataRealShort2 = ByteBuffer.wrap(dataRealInputByteToShort2).order(ByteOrder.LITTLE_ENDIAN).asShortBuffer();
				
				//Transformation from byte buffer to short array.
				bufferDataRealShort1.get(dataRealInputShort1);
				bufferDataRealShort2.get(dataRealInputShort2);				
				
				//Transformation from short array to double array. Right now dataImag1 is not imaginary, it is a part of the wave file just like dataReal.
				for(int i = 0; i < CAPTURE_RATE; i++){

					try{
						dataReal1[i] = dataRealInputShort1[i];
						dataImag1[i] = dataRealInputShort2[i];
					} catch (BufferUnderflowException e){	
						dataReal1[i] = 0;
						dataImag1[i] = 0;
					}						
				}
				
				//Application of the FFT. After this, dataReal will be the real part and dataImag will be the imaginary part of the spectrum.
				mFFT.twoRealFFT(dataReal1, dataImag1, dataReal2, dataImag2);

				//Manipulation of the two pieces of signal.
				for(int i = 0; i < 2; i++){
					
					if (manipulation.getIfManipulated() == true && i == 0){
						
						manipulation.manipulateFftValues(dataReal1, dataImag1);
						dataReal1 = manipulation.returnRealFftValues();
						dataImag1 = manipulation.returnImagFftValues();
					}else if(manipulation.getIfManipulated() == true && i == 1){
						
						manipulation.manipulateFftValues(dataReal2, dataImag2);
						dataReal2 = manipulation.returnRealFftValues();
						dataImag2 = manipulation.returnImagFftValues();
					}
				}
				
				//Possibility to apply window to the spectrum (Hanning, Hamming, Blackman).
//				double[] dataRealW1 = Window.applyHanningWindow(dataReal1);
//				double[] dataRealW2 = Window.applyHanningWindow(dataReal2);
//				double[] dataImagW1 = Window.applyHanningWindow(dataImag1);
//				double[] dataImagW2 = Window.applyHanningWindow(dataImag2);
				
//				dataModul1 = Magnitude.getHalfMagnitude(dataRealW1, dataImagW1);
//				dataModul2 = Magnitude.getHalfMagnitude(dataRealW2, dataImagW2);
				
				//Calculation of the magnitude of the two pieces of data.
				dataModul1 = Magnitude.getHalfMagnitude(dataReal1, dataImag1);
				dataModul2 = Magnitude.getHalfMagnitude(dataReal2, dataImag2);
				
				//Retrieves the position of all the local maxima of the spectrum
				double [] maxPos1 = Tools.findLocalMaxs(dataModul1);
				double [] maxPos2 = Tools.findLocalMaxs(dataModul2);
				
				//Creation of the spectrum's visualization
				double[] spectrum1 = SpectrumManager.createParabolas(dataModul1, maxPos1);
				double[] spectrum2 = SpectrumManager.createParabolas(dataModul2, maxPos2);
				
				//application of the loudness contour
				lc.applyContour(spectrum1);
				lc.applyContour(spectrum2);
				
				//Store of the magnitude for visualization.
				oM.addFftData(spectrum1);
				oM.addFftData(spectrum2);
				
				//Inverse FFT.
				mFFT.ifft(dataReal1, dataImag1, dataRealOutputShort1);
				mFFT.ifft(dataReal2, dataImag2, dataRealOutputShort2);

				//Control of the last N - 1 samples of the data. Here we take care of the overlapping between convolutions.
				for(int i = 0; i < N - 1; i++){
					
					dataRealOutputShort1[i] += lastNShorts[i];
					dataRealOutputShort2[i] += dataRealOutputShort1[CAPTURE_RATE - (N - 1) + i];
					lastNShorts[i] = dataRealOutputShort2[CAPTURE_RATE - (N - 1) + i];
				}
				
				//Writing of the data in the buffer for play-back.
				if(N != 0){
					oM.writeAudio(dataRealOutputShort1,N - 1);
					oM.writeAudio(dataRealOutputShort2,N - 1);
				}else{
					oM.writeAudio(dataRealOutputShort1,0);
					oM.writeAudio(dataRealOutputShort2,0);
				}
				
				//Update of the seek bar.
				seekBar.setProgress((int) (4 * 100d * (double) readingProgress / (double) wavInfo.getSize()));
				
			}//END OF THE LOOP
			
			//Closing the reading.
			try {
				fis1.close();
			} catch (IOException e) {e.printStackTrace();}
		
		//The process is the same for 8 bits samples
		}else if(wavInfo.getBits() == 8){
											
			for(readingProgress = 0; readingProgress < (wavInfo.getSize() / ((Byte.SIZE / 8) * 2)); readingProgress += CAPTURE_RATE - N + 1){
				
				if(N == 0 && readingProgress != 0){
					readingProgress -= 1;
				}
				
				while(!manipulation.setActionBarHeight(getActionBar().getHeight()));
				
				if(shouldContinue == false){
					
					try {
						fis1.close();
					} catch (IOException e) {e.printStackTrace();}
					
					break;
				}
				
					
				if(N != manipulation.getFiltersAndEqualizerMaxN()){
					
					N = manipulation.getFiltersAndEqualizerMaxN();
					
					oM.setPlaybackListener(CAPTURE_RATE, visualizerView, N);
					
					synth.setN(N);
					
					dataRealInputByteToByte1 = new byte[CAPTURE_RATE];
					dataRealInputByteToByte2 = new byte[CAPTURE_RATE];
					
					dataRealInputByte1 = new byte[CAPTURE_RATE];
					dataRealInputByte2 = new byte[CAPTURE_RATE];
					
					if(N != 0){
						lastNBytes = new byte[N - 1];
					}
				}
				
				if(isProgressChanged == true){
					
					try {
						fis1.close();
						fis1 = new FileInputStream(Environment.getExternalStorageDirectory().getPath()+ "/Music/" + audioName);
					} catch (IOException e1) {e1.printStackTrace();	}
									
					try {
						fis1.skip((long) (readingProgress * 2d));
					} catch (IOException e) {e.printStackTrace();}
					
					isProgressChanged = false;
				}
				
				try {
					if(N != 0){
						fis1.read(dataRealInputByteToByte1, 0, dataRealInputByteToByte1.length - (N - 1));
						fis1.read(dataRealInputByteToByte2, 0, dataRealInputByteToByte2.length - (N - 1));
					}else{
						fis1.read(dataRealInputByteToByte1);
						fis1.read(dataRealInputByteToByte2);
					}
				} catch (IOException e1) {e1.printStackTrace();}
				
				ByteBuffer bufferDataRealByte1 = ByteBuffer.wrap(dataRealInputByteToByte1).order(ByteOrder.LITTLE_ENDIAN);
				ByteBuffer bufferDataRealByte2 = ByteBuffer.wrap(dataRealInputByteToByte2).order(ByteOrder.LITTLE_ENDIAN);					
				bufferDataRealByte1.get(dataRealInputByte1);
				bufferDataRealByte2.get(dataRealInputByte2);
				
				for(int i = 0; i < CAPTURE_RATE; i++){

					try{
						dataReal1[i] = (double) dataRealInputByte1[i];
						dataImag1[i] = (double) dataRealInputByte2[i];
					} catch (BufferUnderflowException e){	
						dataReal1[i] = 0;
						dataImag1[i] = 0;
					}						
				}
				
				mFFT.twoRealFFT(dataReal1, dataImag1, dataReal2, dataImag2);
				
				for(int i = 0; i < 2; i++){
					
					if (manipulation.getIfManipulated() == true && i == 0){
						
						manipulation.manipulateFftValues(dataReal1, dataImag1);
						dataReal1 = manipulation.returnRealFftValues();
						dataImag1 = manipulation.returnImagFftValues();
					}else if(manipulation.getIfManipulated() == true && i == 1){
						
						manipulation.manipulateFftValues(dataReal2, dataImag2);
						dataReal2 = manipulation.returnRealFftValues();
						dataImag2 = manipulation.returnImagFftValues();
					}
				}
						
				dataModul1 = Magnitude.getHalfMagnitude(dataReal1, dataImag1);
				dataModul2 = Magnitude.getHalfMagnitude(dataReal2, dataImag2);
				
				double [] maxPos1 = Tools.findLocalMaxs(dataModul1);
				double [] maxPos2 = Tools.findLocalMaxs(dataModul2);
				
				double[] spectrum1 = SpectrumManager.createParabolas(dataModul1, maxPos1);
				double[] spectrum2 = SpectrumManager.createParabolas(dataModul2, maxPos2);
				
				lc.applyContour(spectrum1);
				lc.applyContour(spectrum2);
					
				oM.addFftData(spectrum1);					
				oM.addFftData(spectrum2);
				
				mFFT.ifft(dataReal1, dataImag1, dataRealOutputByte1);
				mFFT.ifft(dataReal2, dataImag2, dataRealOutputByte2);
				
				for(int i = 0; i < N - 1; i++){
					
					dataRealOutputByte1[i] += lastNShorts[i];
					dataRealOutputByte2[i] += dataRealOutputByte1[CAPTURE_RATE - (N - 1) + i];
					lastNBytes[i] = dataRealOutputByte2[CAPTURE_RATE - (N - 1) + i];
				}
				
				if(N != 0){
					oM.writeAudio(dataRealOutputByte1,N - 1);
					oM.writeAudio(dataRealOutputByte2,N - 1);
				}else{
					oM.writeAudio(dataRealOutputByte1,0);
					oM.writeAudio(dataRealOutputByte2,0);
				}
								
				seekBar.setProgress((int) (2 * 100d * (double) readingProgress / (double) wavInfo.getSize()));
			}
			
			try {
				fis1.close();
			} catch (IOException e) {e.printStackTrace();}
		}
	}
	
	/**
	 * Passes the action to the Manipulation class */
	@Override 
	public boolean onTouchEvent(MotionEvent event){
		
		int action = MotionEventCompat.getActionMasked(event);
	
		switch(action) {
			case (MotionEvent.ACTION_DOWN):
				manipulation.onDown(event);
	    		return true;
	    		
			case (MotionEvent.ACTION_POINTER_DOWN):
				manipulation.onPointerDown(event);
	    		return true;
	    		
		    case (MotionEvent.ACTION_MOVE) :
		    	manipulation.onMove(event);
		    	return true;
		    	
		    case (MotionEvent.ACTION_POINTER_UP):
				manipulation.onPointerUp(event);
	    		return true;
		    	
		    case (MotionEvent.ACTION_UP) :
		    	manipulation.onUp(event);
		    	return true;
		}
		return super.onTouchEvent(event);
	}
	
	/**
	 * Creates the menu with the control buttons and passes it to the manipulation class*/
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		getMenuInflater().inflate(R.menu.audio_player, menu);
		if(wavInfo.getFinish() == false){
			manipulation.setMenu(menu);
		}
		return true;
	}
	
	/**
	 * Called when a menu control button is clicked. This manages the masks of the Manipulation class and the text of the options menu */
	@Override
	public boolean onOptionsItemSelected(MenuItem item) {

		switch(item.getItemId()){
		
			case R.id.action_synthesizer:
				
				if(manipulation.getMask() == 0){
					manipulation.setManipulated(true);
				}
				
				if((manipulation.getMask() & manipulation.getSynthMask()) != manipulation.getSynthMask()){
					manipulation.setMask(manipulation.getSynthMask());
					item.setTitle("synth: U");
				}else{
					manipulation.setMask(- manipulation.getSynthMask());
					item.setTitle("synth");
				}
				
				return true;
				
			case R.id.action_filter:
			
				if(manipulation.getMask() == 0){
					manipulation.setManipulated(true);
				}
				
				if((manipulation.getMask() & manipulation.getFilterMask()) != manipulation.getFilterMask()){
					
					manipulation.setMask(manipulation.getFilterMask());
					item.setTitle("filter: U");
					
					if((manipulation.getMaskFixed() & manipulation.getFilterMask()) == manipulation.getFilterMask()){
						manipulation.setMaskFixed(-manipulation.getFilterMask());
					}
					
				}else if((manipulation.getMaskFixed() & manipulation.getFilterMask()) != manipulation.getFilterMask()){
					manipulation.setMaskFixed(manipulation.getFilterMask());
					manipulation.setMask(-manipulation.getFilterMask());
					item.setTitle("filter: L");
				}
				
				return true;
			
			case R.id.action_equalize:	
				
				if(manipulation.getMask() == 0){
					manipulation.setManipulated(true);
				}
				
				if((manipulation.getMask() & manipulation.getEqualizerMask()) != manipulation.getEqualizerMask()){
					
					manipulation.setMask(manipulation.getEqualizerMask());
					item.setTitle("equal: U");
					
					if((manipulation.getMaskFixed() & manipulation.getEqualizerMask()) == manipulation.getEqualizerMask()){
						manipulation.setMaskFixed(-manipulation.getEqualizerMask());
					}
					
				}else if((manipulation.getMaskFixed() & manipulation.getEqualizerMask()) != manipulation.getEqualizerMask()){
					manipulation.setMaskFixed(manipulation.getEqualizerMask());
					manipulation.setMask(-manipulation.getEqualizerMask());
					item.setTitle("equal: L");
				}
				
				return true;
				
			case R.id.action_reset:
				
				if((manipulation.getMask() & manipulation.getEndMask()) != manipulation.getEndMask() && manipulation.getMask() != 0){
					manipulation.setMask(manipulation.getEndMask());
					item.setTitle("reset: U");
				}else if((manipulation.getMask() & manipulation.getEndMask()) == manipulation.getEndMask() && manipulation.getMask() != 0){
					manipulation.setMask(-manipulation.getEndMask());
					item.setTitle("reset");
				}
				
				return true;	
		}
		
		return super.onOptionsItemSelected(item);
	}
	
	/**
	 * In case back is pressed... */
	@Override
	public void onBackPressed() {
		shouldContinue = false;
		try {
			thread.join();
		} catch (InterruptedException e) {e.printStackTrace();}
		
		super.onBackPressed();
	}
	
	/** 
	 * ...or the application stops for any other reason */
	@Override
	protected void onStop() {
		shouldContinue = false;
		try {
			thread.join();
		} catch (InterruptedException e) {e.printStackTrace();}

		super.onStop();
	}	
}