/**
 *  This file is part of DJFourierv4 developed by Blai Meléndez Catalán under
 	the supervision of prof. Horst Eidenberger (TU Wien).

    DJFourierv4 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DJFourierv4 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

 	<http://www.gnu.org/licenses/>. */

package com.example.djfourierv4;

import java.util.ArrayList;

import android.content.Context;
import android.media.AudioFormat;
import android.media.AudioManager;
import android.media.AudioTrack;
import android.media.AudioTrack.OnPlaybackPositionUpdateListener;
import android.util.Log;

public class OutputManager {
	
	private AudioTrack audioTrack;
	
	private int audioFormatChannels;
	
	private int audioFormatBits;
	
	private ArrayList<double[]> fftDataList = new ArrayList<double[]>();

	private int CAPTURE_RATE, N, newN, bufferMin = 0;
	private int size;
		
	public OutputManager(Context context, int channels, int sampleRate, int bits){
		
		//Preparation of the AudioTrack for the play-back with the information extracted from the header of the wave file.
		if(channels == 1){
			audioFormatChannels = AudioFormat.CHANNEL_OUT_MONO;
		}
		
		if(bits == 16){
			audioFormatBits = AudioFormat.ENCODING_PCM_16BIT;
		}else{
			audioFormatBits = AudioFormat.ENCODING_PCM_8BIT;
		}
		
		//Definition of the buffer. It must be short for immediacy.
		bufferMin = 2 * AudioTrack.getMinBufferSize(sampleRate, audioFormatChannels, audioFormatBits);//MODIFIED FOR FREQ. INTERPOL.
		
		Log.d("bufferMin", "bufferMin = " + bufferMin);
		
		audioTrack = new AudioTrack(AudioManager.STREAM_MUSIC, sampleRate, audioFormatChannels, audioFormatBits, bufferMin, AudioTrack.MODE_STREAM);		
	}
	
	/**
	 * Writes the data in the AudioTrack for 8 bits samples */
	public void writeAudio(byte[] data, int nBytes){
		
		audioTrack.write(data, 0, CAPTURE_RATE - nBytes);
	}
	
	/**
	 * Writes the data in the AudioTrack for 16 bits samples */
	public void writeAudio(short[] data, int nShorts){

		audioTrack.write(data, 0, CAPTURE_RATE - nShorts);
	}
	
	/**
	 * Starts playing the data. In STREAM_MODE it waits until the buffer is full */
	public void playAudio(){

		audioTrack.play();
	}
	
	/**
	 * Stores the magnitude of the spectrum i a list for visualization*/
	public void addFftData(double[] fftData){
		
		fftDataList.add(fftData);
	}
	
	/**
	 * Return size of the visualization list*/
	public int getFFTDataListSize(){
		
		return fftDataList.size();
	}
	
	/**
	 * Controls the synchronization between the audio that we are hearing and the spectrum that we are visualizing */
	public void setPlaybackListener(int captureRate, final VisualizerView visualizerView, int n){
		
		CAPTURE_RATE = captureRate;
		
		newN = n;
		
		if(N != 0){
			if(audioFormatBits == 2){
				size = (bufferMin / Short.SIZE) / (CAPTURE_RATE - N + 1) ;
			}else if(audioFormatBits == 3){
				size = (bufferMin / Byte.SIZE) / (CAPTURE_RATE - N + 1) ;
			}
		}else if(N == 0){
			if(audioFormatBits == 2){
				size = (bufferMin / Short.SIZE) / (CAPTURE_RATE);
			}else if(audioFormatBits == 3){
				size = (bufferMin / Byte.SIZE) / (CAPTURE_RATE);
			}
		}
		
		//We set the value of the period of the notification. This is only useful for the first time we call this method. 
		if(N == 0){
			audioTrack.setPositionNotificationPeriod(captureRate);
		}
		
		//AudioTrack will call this every time it reads "Period" number of samples.
		audioTrack.setPlaybackPositionUpdateListener(new OnPlaybackPositionUpdateListener() {
            @Override
            public void onPeriodicNotification(AudioTrack track) {
            	
            	
            	
            	//Visualization of the spectrum.
            	if(fftDataList.size() > 0){
            		visualizerView.callInvalidateFft(fftDataList.remove(0));
            	}
            	
            	controlSize();	
            }
            @Override
            public void onMarkerReached(AudioTrack track) {
               
            }
        });		
	}
	
	/**
	 * Controls the moment when the notification period must be changed */
	public void controlSize(){
		
		//When there are still spectrums with the old N
		if(size > 0){
    		size--;
    	
    	//When there are no more spectrums with the old N	
    	}else if(size == 0){
    		
			N = newN;
			if(N != 0){
				audioTrack.setPositionNotificationPeriod(CAPTURE_RATE - N + 1);
			}else{
				audioTrack.setPositionNotificationPeriod(CAPTURE_RATE);
			}
			size = -1;
		}
	}
}




