/**
 *  This file is part of DJFourierv4 developed by Blai Meléndez Catalán under
 	the supervision of prof. Horst Eidenberger (TU Wien).

    DJFourierv4 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DJFourierv4 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

 	<http://www.gnu.org/licenses/>. */

package com.example.djfourierv4;

import java.text.DecimalFormat;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Paint.Align;
import android.graphics.Paint.Join;
import android.graphics.Path;
import android.util.AttributeSet;
import android.view.View;

public class VisualizerView extends View{
		
	private int CAPTURE_RATE, sampleRate, bps;
	
	private double max, lineXPos, lineYPos;
	
	private double[] fftCoefficients;
	
	private String[] frequencyText = {"31,5 Hz", "63 Hz", "125 Hz", "250 Hz", "500 Hz", "1 kHz", "2 kHz", "4 kHz", "8 kHz", "16 kHz"};
	private double[] frequencies = {31.5, 63, 125, 250, 500, 1000, 2000, 4000, 8000, 16000};
	private String[] frequencyText2 = new String[10];
	private String[] dBFSText16 = {"-20 dB", "-40 dB", "-60 dB", "-80 dB"};
	private String[] dBFSText8 = {"-10 dB", "-20 dB", "-30 dB", "-40 dB"};
	
	private boolean clean = false, isPaintLine = false;
	
	private Paint paint, textFrequencyPaint, linePaint, textdBPaint, gridPaint;
	private Path path;

	public VisualizerView(Context context, int captureRate, int width) {
		super(context);
	}
	

	public VisualizerView(Context context, AttributeSet attrs, int defStyleAttr) {
		super(context, attrs, defStyleAttr);
	}



	public VisualizerView(Context context, AttributeSet attrs) {
		super(context, attrs);
	}
	
	/**
	 * Sets the necessary parameters */
	public void setParameters(int captureRate, int width, int sampleRate, int bps){
		CAPTURE_RATE = captureRate;
		this.sampleRate = sampleRate;
		this.bps = bps;
		fftCoefficients = new double[width];
		setBackgroundColor(Color.TRANSPARENT);
		
		
		paint = new Paint();
		paint.setColor(Color.RED);
		paint.setStrokeWidth(5);
		paint.setStrokeJoin(Join.ROUND);
		paint.setStyle(Paint.Style.STROKE);
		paint.setStrokeCap(Paint.Cap.ROUND);
		
		linePaint = new Paint();
		linePaint.setColor(Color.YELLOW);
		linePaint.setStrokeWidth(2);
		
		gridPaint = new Paint();
		gridPaint.setColor(Color.GRAY);
		gridPaint.setAlpha(128);
		linePaint.setStrokeWidth(1);
		
		textFrequencyPaint = new Paint();
		textFrequencyPaint.setColor(Color.WHITE);
		textFrequencyPaint.setTextSize(20);
		
		textdBPaint = new Paint();
		textdBPaint.setColor(Color.WHITE);
		textdBPaint.setTextSize(20);
		textdBPaint.setTextAlign(Align.RIGHT);
		
		path = new Path();
		
		for(int i = 0; i < frequencyText2.length; i++){
			String c = new DecimalFormat("#.#").format(sampleRate * (frequencies[i] / 44100d)) + " Hz";
			frequencyText2[i] = c;
		}
		
	}
	
	/**
	 * Sets the values of the spectrum that must be painted on the canvas and calls onDraw() */
	public void callInvalidateFft(double[] fft){
		fftCoefficients = fft;
		
		invalidate();
	}
	
	/**
	 * Clears the VisualizerView */
	public void cleanVisualizerView(){
		clean = true;
		invalidate();
	}
	
	/**
	 * Sets where should the yellow lines be painted when there is a down action */
	public void setPaintLine(boolean b, double frequency, double y){
		
		isPaintLine = b;
		lineXPos = 128d * (Math.log(frequency / (20d * ((double) sampleRate / 44100d))) / Math.log(2));
		lineYPos = y;
	}
	
	/**
	 * Sets where should the yellow lines when there si a move action*/
	public void moveLine(double frequency, double y){
		lineXPos = 128d * (Math.log(frequency / (20d * ((double) sampleRate / 44100d))) / Math.log(2));
		lineYPos = y;
	}
	
	/**
	 * Draws on the canvas */
	@Override
	protected void onDraw(Canvas canvas) {
		super.onDraw(canvas);

		if(clean==false){
			
			//Draws the spectrum
			drawFftPath(canvas, fftCoefficients);
			
			//
			for(int i = 0; i < 10; i++){
				
				//Paints the grid
				if(i > 0 || i < 9){
					canvas.drawLine((float) getWidth() / 20f + 20f + ((float) getWidth() / 10f) * i, 0, (float) getWidth() / 20f + 20f + ((float) getWidth() / 10f) * i, (float) (0.95 * (double) getHeight()), gridPaint);
				}
				
				//Paints the frequency values of the frequency axis
				if(sampleRate != 44100){
					canvas.drawText(frequencyText2[i], (float) getWidth() / 20f - 10f + ((float) getWidth() / 10f) * i, (float) 0.98 * getHeight(), textFrequencyPaint);
				}else{
					canvas.drawText(frequencyText[i], (float) getWidth() / 20f - 10f + ((float) getWidth() / 10f) * i, (float) 0.98 * getHeight(), textFrequencyPaint);
				}
			}
			
			//Paints the dB values of the dB axis
			for(int i = 0; i < 4; i++){
				canvas.drawLine(0, (i + 1) * ((float) getHeight() / 5f), getWidth(), (i + 1) * ((float) getHeight() / 5f), gridPaint);
				if(bps == 16){
					canvas.drawText(dBFSText16[i], (float) getWidth() / 20f, (i + 1) * ((float) getHeight() / 5f) - 10f, textdBPaint);
				}else{
					canvas.drawText(dBFSText8[i], (float) getWidth() / 20f, (i + 1) * ((float) getHeight() / 5f) - 10f, textdBPaint);
				}
			}
			
			//Paints the yellow lines that indicate frequency and amplitude of the position of your finger
			if(isPaintLine == true){
				canvas.drawLine((float) lineXPos,(float) (getHeight()), (float) lineXPos, 0f, linePaint);
				canvas.drawLine(0, (float) lineYPos, getWidth(), (float) lineYPos, linePaint);
			}
			
		}else{
			canvas.drawColor(Color.TRANSPARENT);
			clean = false;
		}
	}
	
	/**
	 * Draws the path corresponding to the spectrum */
	public void drawFftPath (Canvas canvas, double[] fft){		
		
		int[] nonZero = Tools.findNonZero(fft);
		
		double point;
		
		if(fft != null && nonZero.length > 0 && nonZero != null){
			
			//Saves the maximum value that the spectrum can reach (depends on the bits per sample)
			if(bps == 16){
				max = 20 * Math.log10((double) Short.MAX_VALUE * 2d);
			}else{
				max = 20 * Math.log10((double) Byte.MAX_VALUE * 2d);
			}
			
			//Computes the value of the starting point
			point = 20 * Math.log10(4d * fft[0] / (double) CAPTURE_RATE);
			
			//Sets the starting point of the path
			path.moveTo(0, getHeight() - (float) (getHeight() * (point / max)));
			
			//Creates the path going through every point
			for(int j = 0; j < nonZero.length; j++){
		
				if(j == 0 && nonZero[j] == 0){
					j++;
				}
				
				point = 20 * Math.log10(4d * fft[nonZero[j]] / (double) CAPTURE_RATE);
				
					path.lineTo((float) (Math.log(nonZero[j] * ((double) sampleRate / (2d * (double) fft.length)) / 20d) / Math.log(2d)) * (float) ((double) getWidth() / 10d)
							, getHeight() - (float) (getHeight() * (point / max)) - 20f);			
			}
			
			//Draws path in the canvas
			canvas.drawPath(path, paint);
			
			//resets the path
			path.reset();
		}
	}	
}




