/**
 *  This file is part of DJFourierv4 developed by Blai Meléndez Catalán under
 	the supervision of prof. Horst Eidenberger (TU Wien).

    DJFourierv4 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DJFourierv4 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

 	<http://www.gnu.org/licenses/>. */

package com.example.djfourierv4;

import java.io.File;
import java.io.FilenameFilter;
import java.util.ArrayList;
import java.util.Locale;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.os.Environment;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.ScrollView;
import android.widget.Toast;

public class ChooseWaveFile extends Activity {
	
	private ArrayList<Button> button = new ArrayList<Button>();
	
	private ScrollView scrollView;
	private LinearLayout linearLayout;
	
	private String[] list; 

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		
		linearLayout = new LinearLayout(this);
		linearLayout.setOrientation(LinearLayout.VERTICAL);
		
		scrollView = new ScrollView(this);
		
		scrollView.addView(linearLayout);
		
		scrollView.setBackgroundResource(R.drawable.wavebackground);
					
		setContentView(scrollView);
		
		//Collect all wave files from music folder.
		File home = new File(Environment.getExternalStorageDirectory().getPath()+ "/Music");
		
		//Acquires only the wave files.
		FilenameFilter textFilter = new FilenameFilter() {
			public boolean accept(File dir, String name) {
				String lowerCaseName = name.toLowerCase(Locale.getDefault());
				if (lowerCaseName.endsWith(".wav")) {
					return true;
				} else {
					return false;
				}
			}
		};
		
		//creates a list of strings with the names of all the wave files.
		list = home.list(textFilter);
		
		if(list.length == 0){
			Toast.makeText(getApplicationContext(), "No wav files found in the Music folder", Toast.LENGTH_LONG).show();
		}
		
		//Creates a button for every wave file and adds it to the layout.
		for(int i = 0; i < list.length; i++){
			
			Button audioButton = new Button(this);
			audioButton.setText(list[i]);
			
			audioButton.setLayoutParams(new ViewGroup.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT,200));
			button.add(audioButton);
			linearLayout.addView(audioButton);
		}
		
		//Sets the behavior of the button when clicked. 
		for(int i = 0; i < button.size(); i++){
			
			if (button.size() > 0){
				
				//"name" determines which song will we manipulate in the Cycle activity.
				final CharSequence name = button.get(i).getText();
				
				button.get(i).setOnClickListener(new Button.OnClickListener() {  
					public void onClick(View v){
						
							//Prepares the button to jump to the Cycle activity.
							Intent intent = new Intent(getApplicationContext(), Cycle.class);
							intent.putExtra("audioName", name);
							startActivity(intent);
					}
		       	});
			}		
		}
	}
}