/**
 *  This file is part of DJFourierv4 developed by Blai Meléndez Catalán under
 	the supervision of prof. Horst Eidenberger (TU Wien).

    DJFourierv4 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DJFourierv4 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

 	<http://www.gnu.org/licenses/>. */

package com.example.djfourierv4;

import java.util.ArrayList;

public class Equalizer {
	
	private int CAPTURE_RATE, sampleRate;
	
	private boolean isEqualizerReady, isEqualizerCreated = false;
	
	private Filter filter0, filter1, filter2, filter3, filter4, filter5, filter6, filter7, filter8, filter9;
	
	private ArrayList<Filter> FilterList = new ArrayList<Filter>();
		
	private double[] dataRealSum;
	private double[] dataImagSum;
	private double[] dataRealRes;
	private double[] dataImagRes;
	
	private EqualizerView equalizerView;
	
	public Equalizer(int captureRate, int sampleRate, EqualizerView equalizerView){
		
		CAPTURE_RATE = captureRate;
		
		this.sampleRate = sampleRate;
		
		this.equalizerView = equalizerView;
		
		dataRealSum = new double[CAPTURE_RATE];
		dataImagSum = new double[CAPTURE_RATE];
		dataRealRes = new double[CAPTURE_RATE];
		dataImagRes = new double[CAPTURE_RATE];
	}
	
	/**
	 * returns if the equalizer is ready for application */
	public boolean getIfEqualizerReady(){
		return isEqualizerReady;
	}
	
	/**
	 * Modifies if the equalizer is ready */
	public void setEqualizerReady(boolean b){
		 isEqualizerReady = b;
	}
	
	/**
	 * returns if the equalizer has been created */
	public boolean getIfEqualizerCreated(){
		return isEqualizerCreated;
	}
	
	/**
	 * creates all the filters for the equalizer */
	public void creatEqualizer(){
		
		if(isEqualizerCreated == false){
		
			filter0 = new Filter(CAPTURE_RATE, true, false);
			filter0.setFilterType(0);
			filter0.setWp(2 * Math.PI * (0 / (double) sampleRate));
			filter0.setWa(2 * Math.PI * ((double) (40 + (40 - 31.5)) / (double) sampleRate));
			filter0.createFilter();
			filter0.setAmplitude(1);
			FilterList.add(filter0);
	
			filter1 = new Filter(CAPTURE_RATE, true, false);
			filter1.setFilterType(1);
			filter1.setAllW(2 * Math.PI * (63d / (double) sampleRate), 2 * Math.PI * ((double) (80 + (80 - 63)) / (double) sampleRate), 2 * Math.PI * ((40 + (40 - 31.5)) / (double) sampleRate), 2 * Math.PI * (0 / (double) sampleRate));
			filter1.createFilter();
			filter1.setAmplitude(1);
			FilterList.add(filter1);
			
			filter2 = new Filter(CAPTURE_RATE, true, false);
			filter2.setFilterType(1);
			filter2.setAllW(2 * Math.PI * (125 / (double) sampleRate), 2 * Math.PI * ((double) (160 + (160 - 125)) / (double) sampleRate), 2 * Math.PI * ((double) (80 + (80 - 63)) / (double) sampleRate), 2 * Math.PI * (63d / (double) sampleRate));
			filter2.createFilter();
			filter2.setAmplitude(1);
			FilterList.add(filter2);
			
			filter3 = new Filter(CAPTURE_RATE, true, false);
			filter3.setFilterType(1);
			filter3.setAllW(2 * Math.PI * (250d / (double) sampleRate), 2 * Math.PI * ((double) (315 + (315 - 250)) / (double) sampleRate), 2 * Math.PI * ((double) (160 + (160 - 125)) / (double) sampleRate), 2 * Math.PI * (125d / (double) sampleRate));
			filter3.createFilter();
			filter3.setAmplitude(1);
			FilterList.add(filter3);
			
			filter4 = new Filter(CAPTURE_RATE, true, false);
			filter4.setFilterType(1);
			filter4.setAllW(2 * Math.PI * (500d / (double) sampleRate), 2 * Math.PI * ((double) (630 + (630 - 500)) / (double) sampleRate), 2 * Math.PI * ((double) (315 + (315 - 250)) / (double) sampleRate), 2 * Math.PI * (250d / (double) sampleRate));
			filter4.createFilter();
			filter4.setAmplitude(1);
			FilterList.add(filter4);
			
			filter5 = new Filter(CAPTURE_RATE, true, false);
			filter5.setFilterType(1);
			filter5.setAllW(2 * Math.PI * (1000d / (double) sampleRate), 2 * Math.PI * ((double)(1250 + (1250 - 1000)) / (double) sampleRate), 2 * Math.PI * ((double) (630 + (630 - 500)) / (double) sampleRate), 2 * Math.PI * (500d / (double) sampleRate));
			filter5.createFilter();
			filter5.setAmplitude(1);
			FilterList.add(filter5);
			
			filter6 = new Filter(CAPTURE_RATE, true, false);
			filter6.setFilterType(1);
			filter6.setAllW(2 * Math.PI * (2000d / (double) sampleRate), 2 * Math.PI * ((double) (2500 + (2500 - 2000)) / (double) sampleRate), 2 * Math.PI * ((double)(1250 + (1250 - 1000)) / (double) sampleRate), 2 * Math.PI * (1000d / (double) sampleRate));
			filter6.createFilter();
			filter6.setAmplitude(1);
			FilterList.add(filter6);
			
			filter7 = new Filter(CAPTURE_RATE, true, false);
			filter7.setFilterType(1);
			filter7.setAllW(2 * Math.PI * (4000d / (double) sampleRate), 2 * Math.PI * ((double) (5000 + (5000 - 4000)) / (double) sampleRate), 2 * Math.PI * ((double) (2500 + (2500 - 2000)) / (double) sampleRate), 2 * Math.PI * (2000d / (double) sampleRate));
			filter7.createFilter();
			filter7.setAmplitude(1);
			FilterList.add(filter7);
			
			filter8 = new Filter(CAPTURE_RATE, true, false);
			filter8.setFilterType(1);
			filter8.setAllW(2 * Math.PI * (8000d / (double) sampleRate), 2 * Math.PI * ((double) (10000 + (10000 - 8000)) / (double) sampleRate), 2 * Math.PI * ((double) (5000 + (5000 - 4000)) / (double) sampleRate), 2 * Math.PI * (4000d / (double) sampleRate));
			filter8.createFilter();
			filter8.setAmplitude(1);
			FilterList.add(filter8);
			
			filter9 = new Filter(CAPTURE_RATE, true, false);
			filter9.setFilterType(1);
			filter9.setAllW(2 * Math.PI * (16000d / (double) sampleRate), 2 * Math.PI * (22050d / (double) sampleRate), 2 * Math.PI * ((double) (10000 + (10000 - 8000)) / (double) sampleRate), 2 * Math.PI * (8000d / (double) sampleRate));
			filter9.createFilter();
			filter9.setAmplitude(1);
			FilterList.add(filter9);
		}
		
		//Sets the filter to created and ready and the filterView to ready.
		equalizerView.createRects();
		isEqualizerCreated = true;
		isEqualizerReady = true;
		equalizerView.setEqualizerReady(true);
	}
	
	/**
	 * Applies the equalizer */
	public void applyEqualizer(double[] dataReal, double[] dataImag){
		
		dataRealSum = new double[CAPTURE_RATE];
		dataImagSum = new double[CAPTURE_RATE];
		dataRealRes = new double[CAPTURE_RATE];
		dataImagRes = new double[CAPTURE_RATE];
		
		for(int i = 0; i < FilterList.size(); i++){
			
			//Application of every filter.
			FilterList.get(i).applyFilter(dataReal, dataImag, dataRealRes, dataImagRes);
			
			//Sum of the result to the total.
			if(i < FilterList.size() - 1){
				
				for(int j = 0; j < CAPTURE_RATE; j++){
					dataRealSum[j] += dataRealRes[j];
					dataImagSum[j] += dataImagRes[j];
				}
			
			//For the last filter.
			}else{
				
				for(int j = 0; j < CAPTURE_RATE; j++){
					dataReal[j] = dataRealSum[j] + dataRealRes[j];
					dataImag[j] = dataImagSum[j] + dataImagRes[j];
				}	
			}
		}		
	}
	
	/**
	 * Finds the filter with the biggest N */
	public int findBiggestN(){
		
		int N = 0;
		
		if(isEqualizerReady == true){
		
			for (int i = 0; i < FilterList.size(); i++){
				
				if(FilterList.get(i).getFilterN() > N){
					N = FilterList.get(i).getFilterN();
				}
			}
		}
		
		return N;
	}
	
	/**
	 * Modifies the amplitude of on filter */
	public void modifyFilterAmplitude(double k, int i){
		
		FilterList.get(i).setAmplitude(k);
	}
	
	/**
	 * returns the equalizerView */
	public EqualizerView getEqualizerView(){
		return equalizerView;
	}
	
	/**
	 * resets equalizer */
	public void resetEqualizer(){
		
		isEqualizerReady = false;
		equalizerView.setEqualizerReady(false);
		
		for (int i = 0; i < FilterList.size(); i++){
			FilterList.get(i).resetAmplitude();
		}
		equalizerView.cleanEqualizerView();
	}
	
}
