/**
 *  This file is part of DJFourierv4 developed by Blai Meléndez Catalán under
 	the supervision of prof. Horst Eidenberger (TU Wien).

    DJFourierv4 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DJFourierv4 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

 	<http://www.gnu.org/licenses/>. */

package com.example.djfourierv4;

import java.util.ArrayList;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Rect;
import android.util.AttributeSet;
import android.view.View;

public class EqualizerView extends View{
	
	private Paint paintRect, paintLine;
	
	private Rect rect;
	
	private boolean clean, isEqualizerReady = false;
		
	private ArrayList<Rect> RectList = new ArrayList<Rect>();
		
	public EqualizerView(Context context) {
		super(context);		
	}
	
	public EqualizerView(Context context, AttributeSet attrs) {
		super(context, attrs);
	}

	public EqualizerView(Context context, AttributeSet attrs, int defStyleAttr) {
		super(context, attrs, defStyleAttr);
	}
	
	/**
	 * Set the parameters of the equalizerView */
	public void setParameters(int captureRate){
		
		paintRect = new Paint();
		paintRect.setColor(Color.BLUE);
		paintRect.setAntiAlias(true);
		paintRect.setAlpha(90);
		paintLine = new Paint();
		paintLine.setColor(Color.YELLOW);
		paintLine.setAntiAlias(true);
		paintLine.setAlpha(20);
		setBackgroundColor(Color.TRANSPARENT);
	}
	
	/**
	 * Creates the rectangles used in the view */
	public void createRects(){
		
		if(RectList.size() > 0){
			RectList.clear();
		}
		
		for(int i = 0; i < 10; i++){
			
			rect = new Rect();
			rect.set(i * getWidth() / 10, getHeight() / 2, ((i + 1) * getWidth() / 10) - 1, getHeight());
			RectList.add(rect);
		}
		clean = false;
		invalidate();
	}
	
	/**
	 * modifies the state of ready of the view */
	public void setEqualizerReady(boolean b){
		isEqualizerReady = b;
	}

	/**
	 * Modifies the rectangles height */
	public void modifyRectHeight(double newHeight, int i){
		
		RectList.get(i).set((getWidth() / 10) * i, (int) newHeight, (getWidth() / 10) * (i + 1) - 1, getHeight());
		invalidate();
	}
	
	/**
	 * Draws on the canvas */
	@Override
	protected void onDraw(Canvas canvas) {
		super.onDraw(canvas);
				
		if(clean == false && isEqualizerReady == true){
			for(int i = 0; i < RectList.size(); i++){
				canvas.drawRect(RectList.get(i), paintRect);
			}
		}else if(clean == true && isEqualizerReady == true){
			canvas.drawColor(Color.TRANSPARENT);
			clean = false;
		}
	}
	
	/**
	 * Finds which equalizer bar should be changed */
	public int findCorrectEqualizerBar(double x){
		
		int i = 0;
		
		if(x < getWidth() / 10){
			return i;
		}else if(x < 2 * getWidth() / 10){
			return i = 1;
		}else if(x < 3 * getWidth() / 10){
			return i = 2;
		}else if(x < 4 * getWidth() / 10){
			return i = 3;
		}else if(x < 5 * getWidth() / 10){
			return i = 4;
		}else if(x < 6 * getWidth() / 10){
			return i = 5;
		}else if(x < 7 * getWidth() / 10){
			return i = 6;
		}else if(x < 8 * getWidth() / 10){
			return i = 7;
		}else if(x < 9 * getWidth() / 10){
			return i = 8;
		}else{
			return i = 9;
		}
	}
	
	/**
	 * clears the view */
	public void cleanEqualizerView(){
		clean = true;
		invalidate();
	}
}
