/**
 *  This file is part of DJFourierv4 developed by Blai Meléndez Catalán under
 	the supervision of prof. Horst Eidenberger (TU Wien).

    DJFourierv4 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DJFourierv4 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

 	<http://www.gnu.org/licenses/>. */

package com.example.djfourierv4;

import java.text.DecimalFormat;

import android.widget.TextView;

public class Filter{

	private int filterType, CAPTURE_RATE, sampleRate, M, N = 0;
	private int sign = 1;
	
	private double wo, wc, wc2, wp, wp2, wa, wa2, k, d, A, m, Aw, B, io1, io2;
	
	private double[] filterReal;
	private double[] filterImag;
	
	private FilterView filterView;
	
	private TextView textView;
	
	private boolean isWoSet = false, isWpSet = false, isInteracting = false, isFilterReady = false, isFilterInvalid = false, isFilterForEqualizer = false, isFilterForModulate = false;
	
	private FFT fft;
	
	public Filter(int captureRate, int sampleRate, FilterView filterView, TextView textView){
		
		CAPTURE_RATE = captureRate;
		this.sampleRate = sampleRate;
		
		this.filterView = filterView;
		
		this.textView = textView;
		
		filterReal = new double[captureRate];
		filterImag = new double[captureRate];
		
		fft = new FFT(captureRate);
	}
	
	public Filter(int captureRate, boolean forEqualizer, boolean forModulate){
		
		CAPTURE_RATE = captureRate;
				
		filterReal = new double[captureRate];
		filterImag = new double[captureRate];
		
		fft = new FFT(captureRate);
		
		isFilterForEqualizer = forEqualizer;
		isFilterForModulate = forModulate;
	}
	
	/**
	 * Creates the filter */
	public boolean createFilter(){
		
		if(filterType == 0){
			
			//Setting the frequencies
			if(wp > wa){
				sign = -1;
				wc = ((Math.PI - wp) + (Math.PI - wa)) / 2;
				wo = Math.PI;
			}else{
				sign = 1;
				wc = (wp + wa) / 2;
				wo = 0;
			}
			
			//Setting filter parameters
			d = 0.05;			
			A = -20 * Math.log10(d);			
			Aw = Math.abs(wa - wp);			
			m = Math.ceil((A - 7.95) / (2.285 * Aw));			
			if(m%2 != 0){
				M = (int) m + 1;
			}else{
				M = (int) m;
			}
			N = M + 1;
			
			//Control maximum length of the filter
			if(N > CAPTURE_RATE / 2){
				N = 0;
				isWoSet = false;
				isWpSet = false;
				textView.setText("Filter's order is too high, try again");
				return false;
			}
			
			//...more parameters
			if(A > 50){
				B = 0.1102 * (A - 8.7);
			}else if(A >= 21 && A <= 50){
				B = 0.5842 * Math.pow((A - 21), 0.4) + 0.07886 * (A - 21);
			}else if (A < 21){
				B = 0;
			}
			
			//Calculation of the filter in the time-domain.
			for( int i = 0; i <= N; i++){
				
				//Calculate Bessel functions.
				io1 = calculateIo(B * Math.sqrt(1 - Math.pow((i - M / 2) / (M / 2), 2)));
				io2 = calculateIo(B);
				
				if(i != (N - 1) / 2){
					if(sign == -1){
						filterReal[i] = (Math.cos((i - (N - 1) / 2) * Math.PI)/((i - (N - 1) / 2) * Math.PI)) * Math.sin((i - (N - 1) / 2) * (wc)) * (io1/io2);
					}else{
						filterReal[i] = (1/((i - (N - 1) / 2) * Math.PI)) * Math.sin((i - (N - 1) / 2) * wc) * (io1/io2);
					}
				}else{
					filterReal[i] = (wc / Math.PI) * (io1/io2);
				}
			}
			
			filterImag = new double[CAPTURE_RATE];
			
			//FFT of the filter.
			fft.fft(filterReal, filterImag);
			
			//Control of the FilterView.
			if(isFilterForEqualizer == false && isFilterForModulate == false){
				filterView.setFilter(filterReal, filterImag);
				filterView.setIsFilterReady(true);
			}
			
			//Setting filter to ready.
			isFilterReady = true;
			
			if(sign == 1 && isFilterForEqualizer == false){
				textView.setText("Cut-off frequency = " + new DecimalFormat("#.##").format((double) sampleRate * wc / (2 * Math.PI)) + " Hz");
			}else if(sign == -1 && isFilterForEqualizer == false){
				textView.setText("Cut-off frequency = " + new DecimalFormat("#.##").format((double) sampleRate * (Math.PI - wc) / (2 * Math.PI)) + " Hz");
			}
			return true;
					
		//Same for band-pass filters.
		}else if(filterType == 1){
			
			if(wo < wa && wo < wp && wp < wa){
				
				if(isFilterForEqualizer == false){
					wc = (wp + wa) / 2;
					wp2 = Math.pow(wo, 2) / wp;
					wa2 = Math.pow(wo, 2) / wa;
					wc2 = (wp2 + wa2) / 2;
				}
				if(isFilterForEqualizer == true){
					wc = (wp + wa) / 2;
					wc2 = (wp2 + wa2) / 2;
				}
				d = 0.05;			
				A = -20 * Math.log10(d);			
				Aw = (wa - wp) / 2 + (wp2 - wa2) / 2;			
				m = Math.ceil((A - 7.95) / (2.285 * Aw));			
				if(m%2 != 0){
					M = (int) m + 1;
				}else{
					M = (int) m;
				}
				N = M + 1;
				
				if(N > CAPTURE_RATE / 2){
					N = 0;
					isWoSet = false;
					isWpSet = false;
					textView.setText("Filter's order is too high, try again");
					return false;
				}
				
				if(A > 50){
					B = 0.1102 * (A - 8.7);
				}else if(A >= 21 && A <= 50){
					B = 0.5842 * Math.pow((A - 21), 0.4) + 0.07886 * (A - 21);
				}else if (A < 21){
					B = 0;
				}
				
				for( int i = 0; i <= N; i++){
					
					io1 = calculateIo(B * Math.sqrt(1 - Math.pow((i - M / 2) / (M / 2), 2)));
					io2 = calculateIo(B);
					
					if(i != (N - 1) / 2){
						filterReal[i] = (1/((i - (N - 1) / 2) * Math.PI)) * ((Math.sin((i - (N - 1) / 2) * wc)) - Math.sin((i - (N - 1) / 2) * wc2)) * (io1/io2);
					}else{
						filterReal[i] = ((wc - wc2) / Math.PI) * (io1/io2);
					}
				}
	
				filterImag = new double[CAPTURE_RATE];
				
				fft.fft(filterReal, filterImag);
				
				if(isFilterForEqualizer == false){
					filterView.setFilter(filterReal, filterImag);
					filterView.setIsFilterReady(true);
				}
				
				isFilterReady = true;
			}else{
				isFilterInvalid = true;
				return false;
			}
			
			if(isFilterForEqualizer == false){
				textView.setText("Central frequency = " + new DecimalFormat("#.##").format((double) sampleRate * wo / (2 * Math.PI)) + " Hz" + " / Pass band = " + new DecimalFormat("#.##").format((double) sampleRate * (wa - wa2) / (2 * Math.PI)) + " Hz");
			}
			
			return true;
		}else{
			return false;
		}
	}
	
	/**
	 * Sets the filter type to band-pass */
	public void setFilterType(int type, double wo){
		filterType = type;
		this.wo = wo;
		isWoSet = true;
	}
	
	/**
	 * Sets the filter type to high or low-pass */
	public void setFilterType(int type){
		filterType = type;
		isWoSet = true;
	}
	
	/**
	 * Returns filter type*/
	public int getFilterType(){
		return filterType;
	}
	
	/**
	 * Returns if filter is invalid */
	public boolean getIfFilterIsInvalid(){
		return isFilterInvalid;
	}
	
	/**
	 * Returns wo */
	public double getWo(){
		return wo;
	}
	
	/**
	 * Sets wp and saves it in isWpSet*/
	public void setWp(double wp){
		this.wp = wp;
		isWpSet = true;
	}
	
	/**
	 * Gets wp */
	public double getWp(){
		return wp;
	}
	
	/**
	 * Sets wa */
	public void setWa(double wa){
		this.wa = wa;
	}
	
	public void setAllW(double wp, double wa, double wp2, double wa2){
		this.wa = wa;
		this.wp = wp;
		this.wa2 = wa2;
		this.wp2 = wp2;
	}
	
	/**
	 * Gets wa */
	public double getWa(){
		return wa;
	}
	
	/**
	 * Gets wc */
	public double getWc(){
		return wc;
	}
	
	/**
	 * Gets pass band */
	public double getPassBand(){
		return (wp - wp2);
	}
	
	/**
	 * Set amplitude of the filter */
	public void setAmplitude(double k){
		this.k = k;
	}
	
	/**
	 * Resets amplitude */
	public void resetAmplitude(){
		this.k = 1;
	}
	
	/**
	 * Gets amplitude */
	public double getAmplitude(){
		return k;
	}
	
	/**
	 * Returns if wo is set */
	public boolean getIfWoSet(){
		return isWoSet;
	}
	
	/**
	 * Returns if wp is set */
	public boolean getIfWpSet(){
		return isWpSet;
	}
	
	/**
	 * Sets if wp is set */
	public void setIfWpSet(boolean b){
		isWpSet = b;
	}
	
	/**
	 * gets the value of isInteracting. Used for interaction with filter */
	public boolean getIfInteracting(){
		return isInteracting;
	}
	
	/**
	 * sets the value of isInteracting. Used for interaction with filter */
	public void setIfInteracting(boolean b){
		isInteracting= b;
	}
	
	/**
	 * Returns the N of the filter */
	public int getFilterN(){
		return N;
	}
	
	/**
	 * Gets the state of ready of the filter */
	public boolean getIfFilterIsReady(){
		return isFilterReady;
	}
	
	/**
	 * Sets the state of the ready of the filter */
	public void setIfFilterIsReady(boolean b){
		isFilterReady = b;
	}
	
	/**
	 * Returns FilterView */
	public FilterView getFilterView(){
		return filterView;
	}
	
	/**
	 * Calculates the Bessel function */
	public double calculateIo(double x){
		
		double sum = 1;
		double nextTerm;
		
		for(int k = 1; k < 500; k++){
			
			nextTerm = Math.pow(Math.pow(x / 2, k) / Tools.factorial(k), 2);
			
			sum += nextTerm;
			
			if(nextTerm < Math.pow(10, -12) * sum){
				break;
			}
		}
		
		return sum;
	}
	
	/**
	 * Applies filter */
	public void applyFilter(double[] dataReal, double[] dataImag){
		
		for(int i = 0; i < CAPTURE_RATE; i++){
			double tmp = dataReal[i] * filterReal[i] - dataImag[i] * filterImag[i];
			dataImag[i] = dataImag[i] * filterReal[i] + dataReal[i] * filterImag[i];
			dataReal[i] = tmp;
		}
	}
	
	/**
	 * Applies filter for the equalizer*/
	public void applyFilter(double[] dataReal, double[] dataImag, double[] dataRealRes, double[] dataImagRes){
		
		for(int i = 0; i < CAPTURE_RATE; i++){
			dataRealRes[i] = k * (dataReal[i] * filterReal[i] - dataImag[i] * filterImag[i]);
			dataImagRes[i] = k * (dataImag[i] * filterReal[i] + dataReal[i] * filterImag[i]);
		}
	}
	
	/**
	 * Resets filter */
	public void resetFilter(){
		
		isFilterReady = false;
		if(isFilterForEqualizer == false && isFilterForModulate == false){
			filterView.setIsFilterReady(false);
		}
		isFilterInvalid = false;
		filterReal = new double[CAPTURE_RATE];
		filterImag = new double[CAPTURE_RATE];
		N = 0;
		isWoSet = false;
		isWpSet = false;
		if(isFilterForEqualizer == false && isFilterForModulate == false){
			filterView.cleanFilterView();
		}
		
	}
}
