/**
 *  This file is part of DJFourierv4 developed by Blai Meléndez Catalán under
 	the supervision of prof. Horst Eidenberger (TU Wien).

    DJFourierv4 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DJFourierv4 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

 	<http://www.gnu.org/licenses/>. */

package com.example.djfourierv4;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Path;
import android.graphics.Paint.Join;
import android.util.AttributeSet;
import android.view.View;

public class FilterView extends View {

	private int CAPTURE_RATE, sampleRate;
	
	private double[] modulFilter;
	private double[] maxVect;
	private double max;
	
	private boolean isFilterReady, isFilterDrawn,  clean = false;
	private Paint paint;
	private Path path;
	
	public FilterView(Context context, int captureRate) {
		super(context);
	}
	
	public FilterView(Context context, AttributeSet attrs, int defStyleAttr) {
		super(context, attrs, defStyleAttr);
	}

	public FilterView(Context context, AttributeSet attrs) {
		super(context, attrs);
	}
	
	/**
	 * Set the necessary parameters */
	public void setParameters(int captureRate, int sampleRate){
		
		CAPTURE_RATE = captureRate;
		this.sampleRate = sampleRate;
		setBackgroundColor(Color.TRANSPARENT);
		paint = new Paint();
		paint.setStrokeWidth(2);
		paint.setColor(Color.GREEN);
		paint.setStrokeJoin(Join.ROUND);
		paint.setStyle(Paint.Style.STROKE);
		paint.setStrokeCap(Paint.Cap.ROUND);
		path = new Path();
	}
	
	/**
	 * Sets the coefficients of the spectrum of the filter and calls for drawing*/
	public void setFilter(double[] filterReal, double[] filterImag){
		
		modulFilter = Magnitude.getHalfMagnitude(filterReal, filterImag);
		maxVect = Tools.findMax(modulFilter);
		max = maxVect[0];
		isFilterDrawn = false;
		clean = false;
		invalidate();
	}
	
	/**
	 * Sets filter to ready */
	public void setIsFilterReady(boolean b){
		isFilterReady = b;
	}
	
	/**
	 * Draws on canvas */
	@Override
	protected void onDraw(Canvas canvas) {
		super.onDraw(canvas);
		if(clean == false){
			drawFilter(canvas);
		}else{
			canvas.drawColor(Color.TRANSPARENT);
			clean = false;
		}
	}
	
	/**
	 * Draws the filter on the canvas */
	public void drawFilter(Canvas canvas){
				
		if(isFilterReady == true && isFilterDrawn == false){
						
			path.moveTo(0, (float) getHeight() - (float) (getHeight() * (modulFilter[0] / max)));
			
			for(int i = 1; i < CAPTURE_RATE / 2; i++){
								
				path.lineTo((float) (Math.log(i * ((double) sampleRate / (2d * (double) modulFilter.length)) / 20d) / Math.log(2d)) * (float) ((double) getWidth() / 10d)
						, (float) getHeight() - (float) (getHeight() * (modulFilter[i] / max)));
				
			}
			
			canvas.drawPath(path, paint);
			path.reset();
			isFilterDrawn = true;
		}
	}
	
	/**
	 * Clears the view */
	public void cleanFilterView(){
		clean = true;
		invalidate();
	}
	
}
