/**
 *  This file is part of DJFourierv4 developed by Blai Meléndez Catalán under
 	the supervision of prof. Horst Eidenberger (TU Wien).

    DJFourierv4 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DJFourierv4 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

 	<http://www.gnu.org/licenses/>. */

package com.example.djfourierv4;

public class LoudnessContour {
	
	double[] f = {20, 25 ,31.5 ,40 ,50 ,63 ,80 ,100 ,125 ,160 ,200 ,250 ,315 ,400 ,500 ,630 ,800,1000 ,1250 ,1600 ,2000 ,2500 ,3150 ,4000 ,5000 ,6300 ,8000 ,10000 ,12500};

	double[] af = {0.532, 0.506, 0.480, 0.455, 0.432, 0.409, 0.387, 0.367, 0.349, 0.330, 0.315, 0.301, 0.288, 0.276, 0.267, 0.259, 0.253, 0.250, 0.246, 0.244, 0.243, 0.243, 0.243, 0.242, 0.242, 0.245, 0.254, 0.271, 0.301};

	double[] Lu = {-31.6, -27.2, -23.0 ,-19.1 ,-15.9 ,-13.0 ,-10.3 ,-8.1 ,-6.2 ,-4.5, -3.1, -2.0 , -1.1 , -0.4 ,  0.0  , 0.3,   0.5  , 0.0, -2.7 ,-4.1 ,-1.0,  1.7, 2.5,   1.2 , -2.1  ,-7.1, -11.2, -10.7,  -3.1};
	
	double[] Tf = {78.5,  68.7  ,59.5 , 51.1 , 44.0 , 37.5,  31.5,  26.5 , 22.1 , 17.9, 14.4 ,11.4  ,8.6  , 6.2 ,  4.4 ,  3.0  , 2.2 ,  2.4 ,  3.5 ,  1.7 , -1.3 , -4.2 ,-6.0  ,-5.4 , -1.5 ,  6.0  ,12.6,  13.9,  12.3};

	double[] contour;
	
	int sampleRate;
	
	int width;
	
	public LoudnessContour(int sampleRate, int width){	
		
		this.sampleRate = sampleRate;
		this.width = width;
	}
	
	/**
	 * Applies loudness contour */
	public void applyContour(double[] fft){
				
		double[] contour = createContour(fft);
		
		for(int i = 0; i < fft.length; i++){
		
			fft[i] *= contour[i];
			
		}
	}
	
	/**
	 * Creates the array containing the values that the spectrum has
	 * to be multiplied by when applying loudness contour */
	public double[] createContour(double[] fft){
		
		double[] contour = new double[f.length];
		double[] result = new double[fft.length];
		
		int Ln = 60;
		double Af;
		
		for(int i = 0; i < f.length; i++){
			
			Af = 4.47E-3 * (Math.pow(10, (0.025 * Ln)) - 1.15) + Math.pow(0.4 * Math.pow(10, ((Tf[i]+Lu[i]) / 10d) - 9), af[i]);
			
		    contour[i] = ((10d / af[i]) * Math.log10(Af) - Lu[i] + 94d);
		}
		
		for(int i = 0; i < contour.length; i++){
			
			contour[i] = 1d / contour[i];
		}
		double maxContour = Tools.findMax(contour)[0];
		contour = Tools.normalizeToValue(contour, maxContour, 1);
		
		int j = 1;
		
		for(int i = 0; i < fft.length; i++){
			
			if(i * ((double) sampleRate / (2d * (double) fft.length)) > f[j] && j < 28){
					j++;
			}
								
			result[i] = contour[j - 1];			
		}
		
		return result;
	}
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
}
