/**
 *  This file is part of DJFourierv4 developed by Blai Meléndez Catalán under
 	the supervision of prof. Horst Eidenberger (TU Wien).

    DJFourierv4 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DJFourierv4 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

 	<http://www.gnu.org/licenses/>. */

package com.example.djfourierv4;

import java.text.DecimalFormat;

import android.graphics.Color;
import android.support.v4.view.MotionEventCompat;
import android.view.Menu;
import android.view.MotionEvent;
import android.widget.TextView;

public class Manipulation {
	
	private int CAPTURE_RATE, mask, maskFixed = 0;
	private int actionBarHeight, bps;
	
	private double height;
	
	private int maskEnd = 1;
	private int maskScale = 2;
	private int maskSynth = 4;
	private int maskFilter = 8;
	private int maskEqualizer = 16;
	private int maskModulate = 32;
	
	private double frequencyDOWN, frequencyMOVE, frequencyUP, sampleRate, noteFrequency;
	
	private boolean isManipulated;
	
	private double[] fftRealValues = new double[CAPTURE_RATE];
	private double[] fftImagValues = new double[CAPTURE_RATE];
	
	private double[] pointerDownYArray = new double[10];
	
	private double[] coefficient;
	private double[] coefficients = new double[7];
	
	private VisualizerView visualizerView;
	
	private TextView textView;
	private TextView textViewK;
	private TextView textView3;
	private TextView textViewFilter;
	
	private Filter filter;
	
	private Equalizer equalizer;
	
	private Synthesizer synth;
	private double[] synthReal;
	private double[] synthImag;
		
	private Menu menu;
		
	public Manipulation(int captureRate, double sampleRate, int bps, double height, double width, VisualizerView visualizerView, TextView textView, TextView textViewK, TextView textView3, TextView textViewFilter, Filter filter, Equalizer equalizer, Synthesizer synth){
		
		CAPTURE_RATE = captureRate;
		this.bps = bps;
		this.height = height;
		this.sampleRate = sampleRate;
		this.visualizerView = visualizerView;
		this.filter = filter;
		this.equalizer = equalizer;
		this.synth = synth;
		textView.setTextColor(Color.WHITE);
		textView.setTextSize(15f);
		this.textView = textView;
		textViewK.setTextColor(Color.WHITE);
		textViewK.setTextSize(15f);
		this.textViewK = textViewK;
		textView3.setTextColor(Color.WHITE);
		textView3.setTextSize(15f);
		this.textView3 = textView3;
		textViewFilter.setTextColor(Color.WHITE);
		textViewFilter.setTextSize(25f);
		this.textViewFilter = textViewFilter;
		
		coefficient = new double[(int) height + 48];
		for(int i = 0; i < coefficient.length; i++){
			coefficient[i] = 1 - ((double) i / (height + 48d));
		}
		
		for(int i = 0; i < coefficients.length; i++){
			coefficients[i] = 1;
		}
		
		
		
	}
	
	/**
	 * The mask system is implemented to be able to apply more than one manipulation at the same time. 
	 * Every manipulation has his own mask and additionally there is two more masks: mask and maskFixed. 
	 * mask is for the manipulations that are being applied and that can be interacted with; maskFixed 
	 * is for the manipulations that are being applied but cannot be interacted with. To put a manipulation 
	 * to any of these two categories we have to add their mask to mask or maskFixed. To check if a 
	 * manipulation is being applied we will multiply its mask by mask or maskFixed. */
	
	/**
	 * Gets mask */
	public int getMask(){
		return mask;
	}
	
	/**
	 * Sets mask */
	public void setMask(int sum){
		mask += sum;
	}
	
	/**
	 * Gets maskFixed */
	public int getMaskFixed(){
		return maskFixed;
	}
	
	/**
	 * Sets maskFixed */
	public void setMaskFixed(int sum){
		maskFixed += sum;
	}
	/**
	 * Gets maskScale */
	public int getScaleMask(){
		return maskScale;
	}
	
	/**
	 * Gets maskSynth */
	public int getSynthMask(){
		return maskSynth;
	}
	
	/**
	 * Gets maskFilter */
	public int getFilterMask(){
		return maskFilter;
	}
	
	/**
	 * Gets the value of N, which is the length of the filter in the time-domain */
	public int getFilterN(){
		return filter.getFilterN();
	}
	
	/**
	 * Gets the max N between the filter and equalizer */
	public int getFiltersAndEqualizerMaxN(){
		
		if(filter.getFilterN() > equalizer.findBiggestN()){
			return filter.getFilterN();
		}else{
			return equalizer.findBiggestN();
		}
	}
	
	/**
	 * Gets if filter is ready */
	public boolean getIfFilterIsReady(){
		return filter.getIfFilterIsReady();
	}
	
	/**
	 * Gets maskEqualizer */
	public int getEqualizerMask(){
		return maskEqualizer;
	}
	
	/**
	 * Gets the biggest N of the filters of the equalizer */
	public int getEqualizerN(){
		return equalizer.findBiggestN();
	}
	
	/**
	 * Gets maskModulate */
	public int getModulateMask(){
		return maskModulate;
	}
	
	/**
	 * Gets maskEnd */
	public int getEndMask(){
		return maskEnd;
	}
	
	/**
	 * Sets that the spectrum of the data can be manipulated */
	public void setManipulated(boolean b){
		isManipulated= b;
	}
	
	/**
	 * Gets if the spectrum of the data can be manipulated */
	public boolean getIfManipulated(){
		return isManipulated;
	}
	
	/**
	 * Gets the manipulation menu */
	public void setMenu(Menu menu){
		this.menu = menu;
	}
	
	/**
	 * Gets the value of the height of the action bar */
	public boolean setActionBarHeight(int height){
		
		actionBarHeight = height;
		
		if(actionBarHeight == 56){
			return true;
		}else{
			return false;
		}
	}
	
	/**
	 * Manipulates the data of the spectrum */
	public void manipulateFftValues(double[] fftReal, double[] fftImag){
	
		fftRealValues = fftReal;
		fftImagValues = fftImag;
		
		//SYNTHESIZER		
		if((mask & maskSynth) == maskSynth){
				
			if(synth.getIfApplySynth() == true){
				
				//Gather the position of each finger and calculates the coefficient to apply to each harmonic
				for(int i = 0; i < coefficients.length; i++){
					if(pointerDownYArray[i] != 0){
						coefficients[i] = coefficient[(int) pointerDownYArray[i]];
					}
				}
				
				//Creates signal
				synthReal = synth.catchPieceOfSignal(coefficients);
				synthImag = new double[CAPTURE_RATE];
				
				//Computes FFT of the signal
				synth.transformSignal(synthReal, synthImag);
				
				//Adds synth spectrum to original spectrum
				synth.sumSignal(fftRealValues, fftImagValues, synthReal, synthImag);
			}
		}
		
		//EQUALIZER
		if(equalizer.getIfEqualizerReady() == true){
			equalizer.applyEqualizer(fftRealValues, fftImagValues);
		}
		
		//FILTER
		if(filter.getIfFilterIsReady() == true){
			filter.applyFilter(fftRealValues, fftImagValues);
		}
		
		//MASK = 0
		if(mask == 0 && maskFixed == 0){
			isManipulated = false;
		}
	}
	
	/**
	 * Returns the real part of the modified spectrum */
	public double[] returnRealFftValues(){
		return fftRealValues;
	}
	
	/**
	 * Returns the imaginary part of the modified spectrum */
	public double[] returnImagFftValues(){
		return fftImagValues;
	}
	
	/**
	 * Defines what must be done when a down touch happens */
	public void onDown(MotionEvent event){
		
		//Save the down position. Starts filling the pointerArray.
		pointerDownYArray[event.getPointerId(0)] = event.getY();
		
		//write the frequency corresponding to the down position in the textView.
		frequencyDOWN = Math.pow(2, (double) event.getX() / 128d) * 20d * (sampleRate / 44100d);
		textView.setText(new DecimalFormat("#.##").format(frequencyDOWN) + " Hz");
		
		//write the amplitude corresponding to the down position in the textView.
		if(bps == 16){
			textView3.setText(new DecimalFormat("#.##").format(-100f * (event.getY() - actionBarHeight) / (height - actionBarHeight)) + " dB");
		}else{
			textView3.setText(new DecimalFormat("#.##").format(-50f * (event.getY() - actionBarHeight) / (height - actionBarHeight)) + " dB");
		}
		
		//paints the yellow lines that cross where you place your finger.
		visualizerView.setPaintLine(true, frequencyDOWN, event.getY() - actionBarHeight);
		
		//SYNTH: Creates the note corresponding to the place you tapped.
		if((mask & maskSynth) == maskSynth && (mask & maskEnd) != maskEnd){
			
			noteFrequency = synth.getNotesFrequency((int) event.getX());
			synth.createNote0(noteFrequency);
		}
		
		//FILTER (This part just takes care of the filter's information textView)
		//APPLICATION OF THE FILTER
		if(((mask & maskFilter) == maskFilter && filter.getIfFilterIsReady() == false && (mask & maskEnd) != maskEnd) && filter.getIfFilterIsInvalid() == false && filter.getIfWoSet() == true){
							
				
			if(filter.getIfWpSet() == false){
				
				textViewFilter.setText("Pass Frequency = " + new DecimalFormat("#.##").format(frequencyDOWN) + " Hz");
			}else{
				
				textViewFilter.setText("Atten. Frequency = " + new DecimalFormat("#.##").format(frequencyDOWN) + " Hz");				
			}
			
		//INTERACTION WITH THE FILTER
		}else if((filter.getIfFilterIsReady() == true && (maskFixed & maskFilter) != maskFilter && (mask & maskEnd) != maskEnd) && filter.getIfFilterIsInvalid() == false){
				
			if(filter.getFilterType() == 0){
				
				if(filter.getIfInteracting() == false){
					
					textViewFilter.setText("Pass Frequency = " + new DecimalFormat("#.##").format(frequencyDOWN) + " Hz");
				}else{
					
					textViewFilter.setText("Atten. Frequency = " + new DecimalFormat("#.##").format(frequencyDOWN) + " Hz");
				}
			}else if(filter.getFilterType() == 1){
				
				if(filter.getIfInteracting() == false){
					
					textViewFilter.setText("Central Frequency = " + new DecimalFormat("#.##").format(frequencyDOWN) + " Hz");
				}else{
					
					if(filter.getIfWpSet() == false){
						
						textViewFilter.setText("Pass Frequency = " + new DecimalFormat("#.##").format(frequencyDOWN) + " Hz");
					}else{
						
						textViewFilter.setText("Atten. Frequency = " + new DecimalFormat("#.##").format(frequencyDOWN) + " Hz");
					}
				}
			}
		}
	}
	
	/**
	 * Defines what must be done when a pointerDown touch happens */
	public void onPointerDown(MotionEvent event){
		
		//Save the subsequent down positions
		pointerDownYArray[MotionEventCompat.getPointerId(event, MotionEventCompat.getActionIndex(event))] =  event.getY();
	}
	
	/**
	 * Defines what must be done when a move touch happens */
	public void onMove(MotionEvent event){
		
		//Controls which finger has moved.
		int pointerCount = event.getPointerCount();
        for(int i = 0; i < pointerCount; i++){
            int pointerIndex = i;
            int pointerId = event.getPointerId(pointerIndex);
    		pointerDownYArray[pointerId] = event.getY(pointerIndex);
        }
        
        //write the frequency corresponding to the move position in the textView.
        frequencyMOVE = Math.pow(2, (double) event.getX() / 128d) * 20d * (sampleRate / 44100d);
		textView.setText(new DecimalFormat("#.##").format(frequencyMOVE) + " Hz");
		
		//write the amplitude corresponding to the down position in the textView.
		if(bps == 16){
			textView3.setText(new DecimalFormat("#.##").format(-100f * (event.getY() - actionBarHeight) / (height - actionBarHeight)) + " dB");
		}else{
			textView3.setText(new DecimalFormat("#.##").format(-50f * (event.getY() - actionBarHeight) / (height - actionBarHeight)) + " dB");
		}
		
		//paints the yellow lines that cross where you place your finger.
		visualizerView.moveLine(frequencyMOVE, event.getY() - 56f);
		
		//SYNTH
		if((mask & maskSynth) == maskSynth && (mask & maskEnd) != maskEnd){
			
			if(noteFrequency != synth.getNotesFrequency((int) event.getX())){
				synth.resetSynth();
				noteFrequency = synth.getNotesFrequency((int) event.getX());
				synth.createNote0(noteFrequency);	
			}
		}
		
		//FILTER (This part just takes care of the filter's information textView)
		//APPLICATION OF THE FILTER
		if(((mask & maskFilter) == maskFilter && filter.getIfFilterIsReady() == false && (mask & maskEnd) != maskEnd) && filter.getIfFilterIsInvalid() == false){
			
			if(filter.getIfWoSet() == false){
				
				textViewFilter.setText("Passband Filter / Central Frequency = " + new DecimalFormat("#.##").format(frequencyMOVE) + " Hz");
			}else if(filter.getIfWpSet() == false){
				
				textViewFilter.setText("Pass Frequency = " + new DecimalFormat("#.##").format(frequencyMOVE) + " Hz");
			}else{
				
				textViewFilter.setText("Atten. Frequency = " + new DecimalFormat("#.##").format(frequencyMOVE) + " Hz");				
			}
		
		//INTERACTION WITH THE FILTER
		}else if((filter.getIfFilterIsReady() == true && (maskFixed & maskFilter) != maskFilter && (mask & maskEnd) != maskEnd) && filter.getIfFilterIsInvalid() == false){
			
			if(filter.getFilterType() == 0){
				
				if(filter.getIfInteracting() == false){
					
					textViewFilter.setText("Pass Frequency = " + new DecimalFormat("#.##").format(frequencyMOVE) + " Hz");	
				}else{
					
					textViewFilter.setText("Atten. Frequency = " + new DecimalFormat("#.##").format(frequencyMOVE) + " Hz");
				}
			}else if(filter.getFilterType() == 1){
				
				if(filter.getIfInteracting() == false){
					
					textViewFilter.setText("Central Frequency = " + new DecimalFormat("#.##").format(frequencyMOVE) + " Hz");
				}else{
					
					if(filter.getIfWpSet() == false){
						
						textViewFilter.setText("Pass Frequency = " + new DecimalFormat("#.##").format(frequencyMOVE) + " Hz");
					}else{
						
						textViewFilter.setText("Atten. Frequency = " + new DecimalFormat("#.##").format(frequencyMOVE) + " Hz");
					}
				}
			}
		}	
	}
	
	/**
	 * Defines what must be done when a up touch happens */
	public void onUp(MotionEvent event){
		
		//Resets the position 0 of the pointerArray.
		pointerDownYArray[MotionEventCompat.getPointerId(event, MotionEventCompat.getActionIndex(event))] = 0;
		
		//write the frequency corresponding to the up position in the textView.
		frequencyUP = Math.pow(2, (double) event.getX() / 128d) * 20d * (sampleRate / 44100d);
		textView.setText(new DecimalFormat("#.##").format(frequencyUP) + " Hz");

		//write the amplitude corresponding to the down position in the textView.
		if(bps == 16){
			textView3.setText(new DecimalFormat("#.##").format(-100f * (event.getY() - actionBarHeight) / (height - actionBarHeight)) + " dB");
		}else{
			textView3.setText(new DecimalFormat("#.##").format(-50f * (event.getY() - actionBarHeight) / (height - actionBarHeight)) + " dB");
		}
		
		//paints the yellow lines that cross where you place your finger.
		visualizerView.setPaintLine(false, frequencyUP, event.getY() - actionBarHeight);
		
		//The action UP is the one that triggers every step of the application of every manipulation except for the synthesizer.
		
		//SYNTH
		if((mask & maskSynth) == maskSynth && (mask & maskEnd) != maskEnd){
			
			synth.resetSynth();
			
		}else if((mask & maskSynth) == maskSynth && (mask & maskEnd) == maskEnd){
			
			mask -= maskSynth;
			synth.resetSynth();
			menu.getItem(0).setTitle("synth");
		}
		
		//FILTER
		//APPLICATION OF THE FILTER
		if(((mask & maskFilter) == maskFilter && filter.getIfFilterIsReady() == false && (mask & maskEnd) != maskEnd) && filter.getIfFilterIsInvalid() == false){
			
			if(filter.getIfWoSet() == false){
				
				//Setting filter type
				if (frequencyDOWN == frequencyUP){
					
					filter.setFilterType(0);
					textViewFilter.setText("Lowpass / Highpass Filter");
				}else{
					
					filter.setFilterType(1, 2 * Math.PI * (frequencyUP / (double) sampleRate));
					textViewFilter.setText("Passband Filter / Central Frequency = " + new DecimalFormat("#.##").format(frequencyUP) + " Hz");
				}
			
			//Setting wp	
			}else if(filter.getIfWpSet() == false){
				
				filter.setWp(2 * Math.PI * (frequencyUP / (double) sampleRate));
				textViewFilter.setText("Pass Frequency = " + new DecimalFormat("#.##").format(frequencyUP) + " Hz");
				
			//Setting wa	
			}else{
				filter.setWa(2 * Math.PI * (frequencyUP / (double) sampleRate));
				filter.createFilter();		
			}
		
		//INTERACTION WITH FILTER
		}else if((filter.getIfFilterIsReady() == true && (maskFixed & maskFilter) != maskFilter && (mask & maskEnd) != maskEnd) && filter.getIfFilterIsInvalid() == false){
			
			//LOW/HIGH-PASS FILTER
			if(filter.getFilterType() == 0){
				
				//Setting wp
				if(filter.getIfInteracting() == false){
					
					filter.setWp(2 * Math.PI * (frequencyUP / (double) sampleRate));
					textViewFilter.setText("Pass Frequency = " + new DecimalFormat("#.##").format(frequencyUP) + " Hz");
					filter.setIfInteracting(true);
				
				//Setting wa and creating filter again	
				}else{
					
					filter.setWa(2 * Math.PI * (frequencyUP / (double) sampleRate));
					filter.resetFilter();
					filter.createFilter();
					filter.setIfInteracting(false);
				}
			
			//BAND-PASS FILTER (idem but setting also wo)
			}else if(filter.getFilterType() == 1){
				
				if(filter.getIfInteracting() == false){
					filter.setFilterType(1, 2 * Math.PI * (frequencyUP / (double) sampleRate));
					textViewFilter.setText("Central Frequency = " + new DecimalFormat("#.##").format(frequencyUP) + " Hz");
					filter.setIfWpSet(false);
					filter.setIfInteracting(true);
					
				}else{
					
					if(filter.getIfWpSet() == false){
						filter.setWp(2 * Math.PI * (frequencyUP / (double) sampleRate));
						textViewFilter.setText("Pass Frequency = " + new DecimalFormat("#.##").format(frequencyUP) + " Hz");
					}else{
						filter.setWa(2 * Math.PI * (frequencyUP / (double) sampleRate));
						filter.resetFilter();
						filter.createFilter();
						filter.setIfInteracting(false);
					}
				}
			}
		
		//REMOVAL OF THE FILTER
		}else if(((mask & maskEnd) == maskEnd && (mask & maskFilter) == maskFilter && (maskFixed & maskFilter) != maskFilter) || filter.getIfFilterIsInvalid() == true){
			
			filter.resetFilter();
			if((mask & maskFilter) == maskFilter){
				mask -= maskFilter;
			}
			if((maskFixed & maskFilter) == maskFilter){
				maskFixed -= maskFilter;
			}
			menu.getItem(1).setTitle("filter");
			textViewFilter.setText("");
		}
		
		//EQUALIZER
		//APPLICATION OF THE FILTER
		if((mask & maskEqualizer) == maskEqualizer && (maskFixed & maskEqualizer) != maskEqualizer && (mask & maskEnd) != maskEnd && equalizer.getIfEqualizerReady() == false){
			
			if(equalizer.getIfEqualizerCreated() == false){
				equalizer.creatEqualizer();
				equalizer.setEqualizerReady(true);
				equalizer.getEqualizerView().setEqualizerReady(true);
				equalizer.getEqualizerView().createRects();
			}else{
				equalizer.setEqualizerReady(true);
				equalizer.getEqualizerView().setEqualizerReady(true);
				equalizer.getEqualizerView().createRects();
				
			}
		
		//INTERACTION WITH THE FILTER
		}else if((mask & maskEnd) != maskEnd && (mask & maskEqualizer) == maskEqualizer && (maskFixed & maskEqualizer) != maskEqualizer && equalizer.getIfEqualizerReady() == true){
			
			equalizer.getEqualizerView().invalidate();
			
			//Calculate new amplitude
			double k = 2 * ((height - actionBarHeight) - (event.getY() - actionBarHeight)) / (height - actionBarHeight);
			
			//Modification of filter amplitude and equalizer bars amplitude
			equalizer.modifyFilterAmplitude(k, equalizer.getEqualizerView().findCorrectEqualizerBar(event.getX()));
			equalizer.getEqualizerView().modifyRectHeight(event.getY() - actionBarHeight, equalizer.getEqualizerView().findCorrectEqualizerBar(event.getX()));
			
			textViewK.setText("k = " + new DecimalFormat("#.##").format(k));
		
		//EQUALIZER LOCKED
		}else if((mask & maskEnd) != maskEnd && (mask & maskEqualizer) != maskEqualizer && (maskFixed & maskEqualizer) == maskEqualizer && equalizer.getIfEqualizerReady() == true){
			
			equalizer.getEqualizerView().cleanEqualizerView();
		
		//REMOVAL OF THE EQUALIZER
		}else if((mask & maskEnd) == maskEnd && (mask & maskEqualizer) == maskEqualizer && (maskFixed & maskEqualizer) != maskEqualizer){
			
			equalizer.resetEqualizer();
			mask -= maskEqualizer;
			menu.getItem(2).setTitle("equal");
			textViewK.setText("");

		}
		
		//RESET
		if((mask & maskEnd) == maskEnd){
			mask -= maskEnd;
			menu.getItem(3).setTitle("reset");
			if(mask == 0 && maskFixed == 0){
				isManipulated = false;
			}
		}
	}
	
	/**
	 * Defines what must be done when a pointerUp touch happens */
	public void onPointerUp(MotionEvent event){
		pointerDownYArray[MotionEventCompat.getPointerId(event, MotionEventCompat.getActionIndex(event))] = 0;
	}
	
	
	
}
