/**
 *  This file is part of DJFourierv4 developed by Blai Meléndez Catalán under
 	the supervision of prof. Horst Eidenberger (TU Wien).

    DJFourierv4 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DJFourierv4 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

 	<http://www.gnu.org/licenses/>. */

package com.example.djfourierv4;


public class Synthesizer {
	
	private int CAPTURE_RATE, sampleRate;
	
	private double max, fundamental, width;
		
	private FFT fft;
	
	private int offset, N = 0;
	private int length = 1;
	
	private double[] noteFrequencies = new double[40];
	private double[] note0, note1, note2, note3, note4, note5;
	
	private boolean applySynth = false;
	
	public Synthesizer(int sampleRate, int captureRate, int bps, double width){
		
		CAPTURE_RATE = captureRate;
		
		this.sampleRate = sampleRate;
		
		this.width = width;
		
		max = Math.pow(2, bps - 1);
		
		fft = new FFT(CAPTURE_RATE);
		
		//Creating the frequencies.
		for(int i = 0; i < 40; i++){
			noteFrequencies[i] = 130.81 * Math.pow(Math.pow(2, 1d/12d), i);
		}
		
	}
	
	/**
	 * Extracts the required number of samples from the created signal*/
	public double[] catchPieceOfSignal(double[] coefficients){
		
		double[] finalSignal = new double[CAPTURE_RATE];
		
		if(N != 0){
			
			for(int i = 0; i < CAPTURE_RATE - (N - 1); i++){
				finalSignal[i] += (1d/6d) * max * coefficients[0] * (coefficients[1] * note0[(i + offset)%note0.length] + coefficients[2] * note1[(i + offset)%note1.length] + coefficients[3] * note2[(i + offset)%note2.length] + coefficients[4] * note3[(i + offset)%note3.length] + coefficients[5] * note4[(i + offset)%note4.length] + coefficients[6] * note5[(i + offset)%note5.length]);
			}
			offset += CAPTURE_RATE - (N - 1);
			
		}else{
			
			for(int i = 0; i < CAPTURE_RATE; i++){
				finalSignal[i] += (1d/6d) * max * coefficients[0] * (coefficients[1] * note0[(i + offset)%note0.length] + coefficients[2] * note1[(i + offset)%note1.length] + coefficients[3] * note2[(i + offset)%note2.length] + coefficients[4] * note3[(i + offset)%note3.length] + coefficients[5] * note4[(i + offset)%note4.length] + coefficients[6] * note5[(i + offset)%note5.length]);
			}
			offset += CAPTURE_RATE;
		}
		
		return finalSignal;
	}
	
	/**
	 * FFT of the created signal */
	public void transformSignal(double[] signalReal, double[] signalImag){
		
		fft.fft(signalReal, signalImag);	
	}
	
	/**
	 * Adds the spectrum of the created signal to the original spectrum */
	public void sumSignal(double[] fftRealValues, double[] fftImagValues, double[] signalReal, double[] signalImag){
		
		for(int i = 0; i < CAPTURE_RATE; i++){
			fftRealValues[i] += signalReal[i];
			fftImagValues[i] += signalImag[i];
		}
	}

	/**
	 * creates fundamental and harmonics */
	public void createNote0(double f){
		
		fundamental = f;
		
		length = 1;
		
		while(Math.cos(2 * Math.PI * (f / (double) sampleRate)) >= Math.cos(2 * Math.PI * (f / (double) sampleRate) * length)){
			length++;
		}
		
		note0 = new double[2 * length];
		
		for(int i = 0; i < length; i++){
			note0[i] = Math.cos(2 * Math.PI * (f / (double) sampleRate) * i);
		}
		
		createNote1();
		createNote2();
		createNote3();
		createNote4();
		createNote5();
		
		applySynth = true;
	}
	
	/**
	 * Creates harmonic number X */
	public void createNote1(){
		
		double f = 2 * fundamental;
		
		length = 1;
		
		while(Math.cos(2 * Math.PI * (f / (double) sampleRate)) >= Math.cos(2 * Math.PI * (f / (double) sampleRate) * length)){
			length++;
		}
		
		note1 = new double[2 * length];
		
		for(int i = 0; i < length; i++){
			note1[i] = Math.cos(2 * Math.PI * (f / (double) sampleRate) * i);
		}
	}

	public void createNote2(){
	
		double f = 3 * fundamental;
		
		length = 1;
		
		while(Math.cos(2 * Math.PI * (f / (double) sampleRate)) >= Math.cos(2 * Math.PI * (f / (double) sampleRate) * length)){
			length++;
		}
		
		note2 = new double[2 * length];
		
		for(int i = 0; i < length; i++){
			note2[i] = Math.cos(2 * Math.PI * (f / (double) sampleRate) * i);
		}
	}

	public void createNote3(){
		
		double f = 4 * fundamental;
		
		length = 1;
		
		while(Math.cos(2 * Math.PI * (f / (double) sampleRate)) >= Math.cos(2 * Math.PI * (f / (double) sampleRate) * length)){
			length++;
		}
		
		note3 = new double[2 * length];
		
		for(int i = 0; i < length; i++){
			note3[i] = Math.cos(2 * Math.PI * (f / (double) sampleRate) * i);
		}
	}

	public void createNote4(){
		
		double f = 5 * fundamental;
		
		length = 1;
		
		while(Math.cos(2 * Math.PI * (f / (double) sampleRate)) >= Math.cos(2 * Math.PI * (f / (double) sampleRate) * length)){
			length++;
		}
		
		note4 = new double[2 * length];
		
		for(int i = 0; i < length; i++){
			note4[i] = Math.cos(2 * Math.PI * (f / (double) sampleRate) * i);
		}
	}

	public void createNote5(){
	
		double f = 6 * fundamental;
		
		length = 1;
		
		while(Math.cos(2 * Math.PI * (f / (double) sampleRate)) >= Math.cos(2 * Math.PI * (f / (double) sampleRate) * length)){
			length++;
		}
		
		note5 = new double[2 * length];
		
		for(int i = 0; i < length; i++){
			note5[i] = Math.cos(2 * Math.PI * (f / (double) sampleRate) * i);
		}
	}
	
	/**
	 * Determines which note should sound depending on the position of the finger */
	public double getNotesFrequency(int position){
		
		double f = 0;
		
		for(int i = 1; i <= width / 32; i++){
			
			if(position < i * 32){
				f = noteFrequencies[i - 1];
				break;
			}	
		}
		
		return f;
	}
	
	/**
	 * Sets N. This parameter is essential to know the number of samples that 
	 * should be extracted from the created signal (CAPTURE_RATE - N + 1) */
	public void setN(int n){
		this.N = n;
	}
	
	/**
	 * Resets the synth */
	public void resetSynth(){
		offset = 0;
		note0 = new double[length];
		applySynth = false;
	}
	
	/**
	 * Get if synth should be applied by the manipulation class */
	public boolean getIfApplySynth(){
		return applySynth;
	}
	
	/**
	 * Set is synth should be applied byt the manipulation class*/
	public void setIfApplySynth(boolean b){
		applySynth = b;
	}
	
}
