/**
 *  This file is part of DJFourierv4 developed by Blai Meléndez Catalán under
 	the supervision of prof. Horst Eidenberger (TU Wien).

    DJFourierv4 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DJFourierv4 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

 	<http://www.gnu.org/licenses/>. */

package com.example.djfourierv4;

public class Tools {
		
	public Tools(){
		
	}

	/** Finds and returns the maximum value of the bytes of the FFT.
	 * 
	 * This is used to normalize in the drawColorLines method*/
	public static double[] findMax(double[] points){
		
		if (points!=null){
        
        double[] result = new double[2];
        
        for (int j = 0; j < points.length; j++) {        	
            if (Math.abs(points[j]) > result[0]) {
                result[0] = Math.abs(points[j]);
                result[1] = j;
            }
        }

        return result;
        
		}else{
			return null;
		}
    }
	
	/**
	 * Finds the local maxima of an array and stores their position into an new array */
	public static double[] findLocalMaxs(double[] fft){
		
		double[] maxPos = new double[fft.length];
		double[] result;
		
		int k = 0;
		
		if(fft[0] >= fft[1]){
			maxPos[k] = 0;
			k++;
		}
		
		for(int i = 1; i < fft.length - 1; i++){
			
			if(fft[i] >= fft[i + 1] && fft[i] >= fft[i -1]){
				maxPos[k] = i;
				k++;
			}
		}
		
		if(fft[fft.length - 1] >= fft[fft.length - 2]){
			maxPos[k] = fft.length - 1;
			result = new double[k];
		}else if(k > 0){
			result = new double[k - 1];
		}else{
			result = new double[0];
		}
		
		for(int i = 0; i < result.length; i++){
			
			result[i] = maxPos[i];
		}
		
		return result;
	}
	
	/**
	 * Normalizes array to value */
	public static double[] normalizeToValue(double[] v, double max, double value){
		
		for(int i = 0; i < v.length; i++){
			
			v[i] = ((v[i] * value) / max);
		}
		
		return v;
	}
	
	/**
	 * Normalizes array to max short value */
	public static double[] normalizeShorts(double[] v, double max){
		
		for(int i = 0; i < v.length; i++){
			
			v[i] = ((v[i] * 32768) / max);
		}
		
		return v;
	}
	
	/**
	 * Normalizes array to max byte value */
	public static double[] normalizeBytes(double[] v, double max){
		
		for(int i = 0; i < v.length; i++){
			
			v[i] = ((v[i] * 128) / max);
		}
		
		return v;
	}
	
	/**
	 * Calculates the factorial of an integer*/
	public static int factorial (int n){
		
	    int res=1;
	    
	    for(int i = 1; i <= n; i++){
	        res *= i;
	    }
	    
	    return res;
	}
	
	/**
	 * Finds the position of the non-zero values of an array */
	public static int[] findNonZero(double[] v){
		
		int[] result;
		
		if(v != null){
		
			int[] tmp = new int[v.length];
			
			int j = 0;
			
			for(int i = 0; i < v.length; i++){
				
				if(v[i] != 0){
					
					tmp[j] = i;
					j++;
				}
			}
			
			int k = 0;
			
			for(int i = 0; i < tmp.length - 2; i++){
				
				if(tmp[i] != 0 || tmp[i + 1] != 0 || tmp[i + 2] != 0){
					k++;
				}			
			}
			
			result = new int[k];
			
			for(int i = 0; i < k; i++){
				
				result[i] = tmp[i];
			}
			
		}else{
			
			return null;
		}
		
		return result;
	}	
}
